<?php

/**
 * @file
 * Functions and classes for the all entities.
 */

define('LIMESURVEY_SYNC_SYNCHRONIZED', 1);
define('LIMESURVEY_SYNC_DESYNCHRONIZED', 2);
define('LIMESURVEY_SYNC_UNSYNCHRONIZED', 3);

// Fieldname for LS question type '1' contain an '#" that break Drupal queries.
// I replace it.
define('LIMESURVEY_SYNC_REPLACE_HASK_KEY', 'z');

/**
 * Class managing building dynamic entities.
 *
 * Building entity_info, entity_property_info, schema, entity default values and
 * updating entity table if needed. Take care of Drupal fields, LimeSurvey fixed
 * fields and Limesurver variables fields.
 */
class LimesurveySyncEntityInfo extends CustomizableEntitiesEntityInfo {
  // 'survey', or 'response' or 'answer' or 'token'.
  protected $lss_entity_type;
  protected $module = 'limesurvey_sync';

  // the entity contains 'lss_sync' synchronized field.
  protected $user_col = TRUE;
  protected $status_col = TRUE;
  protected $title_col = TRUE;
  protected $date_col = TRUE;
  protected $language_col = TRUE;
  protected $comment_col = FALSE;
  protected $promote_col = FALSE;
  protected $sticky_col = FALSE;
  // The LS primary key.
  protected $ls_key = '';
  // The LS storing date.
  protected $ls_date_key = '';

  // Uri for the overview page.
  public $root_admin_ui = 'admin/structure/limesurvey';
  // Uri for the view page.
  public $crud_path = 'limesurvey';

  // If the entity manage revision.
  protected $revision = TRUE;
  // If the entity is fieldable.
  protected $fieldable = TRUE;
  // If the entity is exportable.
  protected $exportable = FALSE; // --> add manually 'rules controller class' in entity_info.
  // Correspond to entity_info['bundle of'].

  /**
   * Set the new object.
   */
  public function __construct($mymodule = NULL, $source = NULL, $parent_class = NULL, $source_class = NULL) {
    if (empty($this->entityname)) {
      $this->entityname = limesurvey_sync_api_entityname($this->lss_entity_type);
    }
    if (empty($this->crud_path)) {
      $this->crud_path = limesurvey_sync_api_typename($this->lss_entity_type);
    }

    $this->customizable_entities_entry['lss_entity_type'] = $this->lss_entity_type;

    $for_classes = drupal_ucfirst(str_replace('_', '', $this->lss_entity_type));
    // Set classes.
    $classes = array(
      'entityclassname' => 'LimesurveySyncEntity',
      'controller_class' => 'LimesurveySyncController',
      'ui_controller_class' => 'LimesurveySyncUIController',
      'install_class' => 'LimesurveySyncInstall',
      'metadata_controller_class' => 'LimesurveySyncMetadataController',
      'extra_fields_controller_class' => 'LimesurveySyncExtraFieldsController',
    );
    foreach ($classes as $key => $class) {
      if (empty($this->$key)) {
        if (class_exists("{$class}{$for_classes}")) {
          $this->$key = "{$class}{$for_classes}";
        }
        else {
          $this->$key = $class;
        }
      }
    }

    // Set custom view modes
    $this->view_modes = array(
      'full' => array(
        'label' => t('Full'),
        //'custom settings' => FALSE,
      ),
      'admin_dp' => array(
        'label' => t('Admin Drupal'),
        'custom settings' => TRUE,
      ),
      'admin_ls' => array(
        'label' => t('Admin LimeSurvey'),
        'custom settings' => TRUE,
      ),
    );

    parent::__construct($mymodule, $source, $parent_class, $source_class);
  }

  /** Inform the system about one or more entity types
   *
   * Building entity_info corresponding to settings above.
   *
   * @see hook_entity_info()
   */
  public function entity_info() {

    $entities_info = parent::entity_info();

    if (!empty($entities_info)) {
      if ($this->ls_key) {
        // Add custom entries : corresponding LS primary key.
        $entities_info[$this->entityname]['entity keys']['ls key'] = $this->ls_key;
      }
      if ($this->ls_date_key) {
        // Add custom entries : LS primary last change date field.
        $entities_info[$this->entityname]['entity keys']['ls date key'] = $this->ls_date_key;
      }
    }

    return $entities_info;
  }


}

class LimesurveySyncInstall extends CustomizableEntitiesInstall {

  protected $ls_sid = NULL;
  protected $lss_entity_type = NULL;

  public function __construct($entity_type, $entity_infos) {
    parent::__construct($entity_type, $entity_infos);
    $this->ls_sid = (!empty($this->entity_info['customizable entities']['ls_sid'])) ? $this->entity_info['customizable entities']['ls_sid'] : NULL;
    $this->lss_entity_type = (!empty($this->entity_info['customizable entities']['lss_entity_type'])) ? $this->entity_info['customizable entities']['lss_entity_type'] : FALSE;
  }

  /**
   * Provide hook_permission for the entity.
   *
   * @return
   *   An array, the entity permissions.
   *
   * @see hook_permission()
   */
  public function permission($datas = array()) {
    $permissions = parent::permission($datas);
    // Add permission to synchronize DP to ls.
    if (!in_array($this->lss_entity_type, array('survey')) && $perm_dp2ls = limesurvey_sync_api_permission_item($this->entity_type, 'dp2ls')) {
      $permissions[$perm_dp2ls] = array(
        'title' => t('Synhronize @label from Drupal to LimeSurvey', array('@label' => $this->entity_info['plural label'])),
        'description' => t('Modify datas into the LimeSurvey database from this Drupal site'),
        'restrict access' => TRUE,
      );
    }
    return $permissions;
  }

  /**
   * Define schema for custom fields.
   *
   * each class should implement it.
   *
   * @return
   *   An array, the custom schema fields or an empty array if there is
   *   no fields.
   *
   * @see LimesurveySyncDpInfos::schema()
   */
  protected function schema_custom_fields($revision = NULL) {
    $fields = array();
    // Add custom entries for synchronization monitoring.
    $fields['lss_sync'] = array(
      'type' => 'int',
      'unsigned' => TRUE,
      'not NULL' => TRUE,
      'default' => 1,
      'size' => 'tiny',
      'description' => "Sync status of this {$this->entity_info['label']} (1 for synchronized, 2 for desynchronized, 3 for unsynchronized).",
    );
    $fields['lss_last_sync'] = array(
      'description' => "The Unix timestamp when the {$this->entity_info['label']} was most recently synchronized.",
      'type' => 'int',
      'not null' => TRUE,
      'default' => 0,
    );
    $fields['lss_last_way'] = array(
      'description' => "Last synchronization direction for the {$this->entity_info['label']}.",
      'type' => 'text',
      'not null' => FALSE,
      'default' => NULL,
    );

    $fields += parent::schema_custom_fields($revision);


    return array_merge($fields, $this->schema_ls_fields());
  }

  /**
   * Define schema for LS fields.
   *
   * Merge known and variable LS fields schema, returning all LS fields schema.
   * If a LS field exists on LS but is not defined on Drupal, provide a watchdog
   * information about it.
   *
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array, the exhaustive LS schema fields or an empty array if there is
   *   no LS fields.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_ls_fields() {

    // Set here for performance raisons. Thoses tables don't need update.
    $lss_entity_type = $this->entity_info['customizable entities']['lss_entity_type'];
    if (empty($this->ls_sid)) {
      return array();
    }
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = limesurvey_sync_ls_api_retrieve_ls_fields($lss_entity_type, $this->ls_sid);
    // If the token table does not exists on limesurvey, return nothing.
    $ls_current_fields = $existing_ls_fields;
    $fields_schema = array();
    foreach ($ls_current_fields as $key => $current_ls_field) {
      $field_schema = $this->schema_ls_fields_known($current_ls_field);
      if ($field_schema) {
        $fields_schema[$current_ls_field] = $field_schema;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      foreach ($remaining_fields as $key => $variable_ls_field) {

        $field_schema = $this->schema_ls_fields_variables($variable_ls_field, $this->ls_sid);
        if ($field_schema) {
          $fields_schema[$variable_ls_field] = $field_schema;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !db_table_exists($this->entity_info['base table'])) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $this->ls_sid,
        '%type' => $this->lss_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey fields %fields have not been imported into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey fields %fields have not been imported into Drupal (from %type %sid).', $arg), 'warning', FALSE);
    }

    return $fields_schema;
  }

  /**
   * Define schema for a fixed (known) LS field.
   *
   * For token : Handled tid, completed, firstname, etc... fields and for answer
   * handled id, lastpage, startlanguage, etc.. fields.
   *
   * @param $field
   *   A string, the LS field name.
   *
   * @return
   *   An array, the schema field or FALSE if it is not a fixed field.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_ls_fields_known($field) {
    return FALSE;
  }

  /**
   * Return the corresponding Drupal database schema of a LimeSurvey field.
   *
   * This fonction handles LimeSurvey token or answer fields that are specific to
   * a particular LimeSurvey survey. For the token table it corresponds to the
   * attribute fields, and for the answer table it corresponds to the questions
   * fields. The LimeSurvey datetime fields will be converted to timestamp on
   * Drupal.
   *
   * @param $field
   *   A string, the LS field name.
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array of Drupal schema corresponding to the provided LimeSurvey field or
   *   FALSE if the provided field is not a variable LimeSurvey field.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_ls_fields_variables($field, $sid) {
    return FALSE;
  }

  protected function bundle_settings_default_order($extra, $view_mode = 'default') {
    $order = parent::bundle_settings_default_order($extra, $view_mode);
    $order['lss_sync'] = array($view_mode => TRUE);
    $order['lss_status'] = array($view_mode => TRUE);
    $order['lss_last_sync'] = array($view_mode => TRUE);
    $order['lss_last_way'] = array($view_mode => TRUE);
    $order['lss_survey'] = array($view_mode => TRUE);
    $order['lss_dp_filling_page'] = array($view_mode => TRUE);
    $order = drupal_array_merge_deep($order, $this->bundle_settings_default_order_ls_fields($view_mode));
    // Set many extra fileds to hidden for my custom view_modes.
    $admin_dp = array('lss_sync', 'lss_status', 'lss_last_sync', 'lss_last_way', 'lss_survey', 'lss_dp_filling_page');
    $admin_ls = array('lss_page', 'lss_ls_filling_page');
    foreach ($this->entity_info['bundles'] as $bundle => $bundle_datas) {
      if (isset($extra[$this->entity_type][$bundle]['display'])) {
        foreach ($extra[$this->entity_type][$bundle]['display'] as $field => $extra_field_info) {
          foreach (array('admin_dp', 'admin_ls') as $custom_view_mode) {
            if (!isset($order[$field][$custom_view_mode])) {
              $visibility = ($custom_view_mode == 'admin_ls' && ((isset($order[$field][$custom_view_mode]) && $order[$field][$custom_view_mode]) || in_array($field, $admin_ls))) || ($custom_view_mode == 'admin_dp' && ((isset($order[$field][$custom_view_mode]) && $order[$field][$custom_view_mode]) || in_array($field, $admin_dp)));
              $order[$field][$custom_view_mode] = $visibility;
              if ($visibility) {
                // If visible on admin_ls view_mode, hide it on default view mode.
                $order[$field]['default'] = FALSE;
              }
            }
          }
        }
      }
    }
    // Set lss_page and lss_ls_filling_page at the end.
    foreach (array('lss_page', 'lss_ls_filling_page') as $field) {
      foreach ($order[$field] as $custom_view_mode => $visibility) {
        unset($order[$field][$custom_view_mode]);
        $order[$field][$custom_view_mode] = $visibility;
      }
    }

    return $order;
  }

  protected function bundle_settings_default_order_ls_fields($view_mode = 'default') {
    // Set here for performance raisons. Thoses tables don't need update.
    $lss_entity_type = $this->entity_info['customizable entities']['lss_entity_type'];
    $sid = (!empty($this->entity_info['customizable entities']['ls_sid'])) ? $this->entity_info['customizable entities']['ls_sid'] : NULL;
    if (empty($sid)) {
      return array();
    }
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = array_flip(limesurvey_sync_ls_api_retrieve_ls_fields($lss_entity_type, $sid));
    // If the token table does not exists on limesurvey, return nothing.
    $fields_order = array();
    $known_order = $this->bundle_settings_default_order_ls_fields_known($view_mode);
    // Returns all possible ls known fields in the right order.
    foreach ($known_order as $key => $current_ls_field) {
      if (isset($existing_ls_fields[$key])) {
        // The field does exists on the entity
        $fields_order[$key] = $current_ls_field;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      $variable_order = $this->bundle_settings_default_order_ls_fields_variables($view_mode);
      foreach ($variable_order as $key => $variable_ls_field) {
        if (isset($remaining_fields[$key])) {
          $fields_order[$key] = $variable_ls_field;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !db_table_exists($this->entity_info['base table'])) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $sid,
        '%type' => $lss_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey extra fields %fields have not been correctly set into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey extra fields %fields have not been correctly set into Drupal (from %type %sid).', $arg), 'warning', FALSE);
    }

    // Handle ls_sid and lss_entity_type
    $fields_order['lss_entity_type']['default'] = FALSE;
    if (!empty($this->ls_sid)) {
      $fields_order['ls_sid']['default'] = FALSE;
    }

    return $fields_order;
  }

  protected function bundle_settings_default_order_ls_fields_known($view_mode = 'default') {
    // For returning a field : return array('my_field' => array($view_mode => TRUE));
    return array();
  }

  protected function bundle_settings_default_order_ls_fields_variables($view_mode = 'default') {
    return array();
  }


}

class LimesurveySyncMetadataController extends CustomizableEntitiesMetadataController {

  /**
   * Provided entity_property_info() informations for custom fields.
   *
   * Each class should implement it.
   *
   * @return
   *   An array, the DP entity_property_info() fields.
   *
   * @see LimesurveySyncDpInfos::entity_property_info()
   */
  protected function entityPropertyInfo_custom_fields($fields) {

    // No html for label.
    // set some prefix for sorting :
    // '_ID' for dp and LS ids.
    // 'DP'for drupal fields.
    // 'LS admin for ls admin fields.
    // 'LS datas' then the field name without prefix for answer datas and token
    // attributes.
    // No need to add the survey ID : provided by views.
    $lss_entity_type = $this->info['customizable entities']['lss_entity_type'];
    $sid = (!empty($this->info['customizable entities']['ls_sid'])) ? $this->info['customizable entities']['ls_sid'] : NULL;
    $survey_arg = (!empty($sid)) ? array(
      '@survey' => t(' (survey #@sid)', array(
        '@sid' => $sid,
      )),
    ) : array();
    $arg_label = array_merge(array(
      '@type' => $lss_entity_type,
    ), $survey_arg);
    $survey_desc = (!empty($sid)) ? array(
      '%survey' => t(' (survey #@sid)', array(
        '@sid' => $sid,
      )),
    ) : array();
    $arg_desc = array_merge(array(
      '%type' => $lss_entity_type,
    ), $survey_desc);


    //$fields = parent::entityPropertyInfo_custom_fields($fields);

    // Add the lss_entity_type_field.
    $fields['lss_entity_type'] = array(
      'label' => t('LimeSurvey entity type'),
      'description' => t('The type of LimeSurvey entity (%type)', $arg_desc),
      'type' => 'token',
      'options list' => 'limesurvey_sync_api_lss_entity_type_list',
      'entity views field' => TRUE,
    );

    if (in_array('lss_sync', $this->info['schema_fields_sql']['base table'])) {
      $permission = customizable_entities_permission_item('administer', $this->type);
      $fields['lss_sync'] = array(
        'label' => t('DP synchronization status'),
        'description' => t('sync ID (1 for synchronized, 2 for desynchronized, 3 for unsynchronized)'),
        'type' => 'integer',
        'schema field' => 'lss_sync',
        'options list' => 'limesurvey_sync_api_lss_sync_status_list',
        'setter permission' => $permission,
        'setter callback' => 'entity_property_verbatim_set',
      );
    }
    if (in_array('lss_last_sync', $this->info['schema_fields_sql']['base table'])) {
      $fields['lss_last_sync'] = array(
        'label' => t('Date last sync'),
        'description' => t('The Unix timestamp when the %type was most recently synchronized.', $arg_desc),
        'type' => 'date',
        'getter callback' => 'entity_metadata_customizable_entities_get_date_property',
        'schema field' => 'lss_last_sync',
      );
    }
    if (in_array('lss_last_way', $this->info['schema_fields_sql']['base table'])) {
      $fields['lss_last_way'] = array(
        'label' => t('Last synchronization direction'),
        'description' => t('The synchronization direction %type on the last synchronization.', $arg_desc),
        'type' => 'token',
        'options list' => 'limesurvey_sync_api_lss_last_way_list',
        'schema field' => 'lss_last_way',
      );
    }
    if (!empty($sid)) {
      // Answer and token.
      // Add ref to survey
      $fields['lss_survey'] = array(
        'label' => t('The associated survey'),
        'description' => t('The survey entity associated to this %type', $arg_desc),
        'type' => limesurvey_sync_api_entityname('survey'),
        'getter callback' => 'limesurvey_sync_api_get_lss_survey',
        'access callback' => 'limesurvey_sync_api_entity_metadata_lss_survey_properties_access',
      );
    }

    $fields['lss_page'] = array(
      'label' => t('LimeSurvey admin page'),
      'description' => t('The LimeSurvey admin page entity corresponding to this %type', $arg_desc),
      'type' => 'uri',
      'getter callback' => 'limesurvey_sync_api_get_lss_page',
    );
    $fields['lss_dp_filling_page'] = array(
      'label' => t('Starting response from drupal'),
      'description' => t('The Drupal page for starting the response', $arg_desc),
      'type' => 'uri',
      'getter callback' => 'limesurvey_sync_api_get_lss_dp_filling_page',
    );
    $fields['lss_ls_filling_page'] = array(
      'label' => t('Starting response from LimeSurvey'),
      'description' => t('The LimeSurvey page for starting the response', $arg_desc),
      'type' => 'uri',
      'getter callback' => 'limesurvey_sync_api_get_lss_ls_filling_page',
    );

    // retrieve LS fields propereties infos.
    $ls_fields_properties = $this->entityPropertyInfo_ls_fields($sid);
    // Set a specific permission (synchronize dp2ls) for editing them.
    $ls_field_edit_perm = limesurvey_sync_api_permission_item($this->type, 'dp2ls');
    foreach ($ls_fields_properties as $field => $datas) {
      if (!isset($datas['setter permission'])) {
        $ls_fields_properties[$field]['setter permission'] = $ls_field_edit_perm;
      }
    }
    $fields = array_merge($fields, $ls_fields_properties);
    $fields = parent::entityPropertyInfo_custom_fields($fields);
    return $fields;
  }

  /**
   * Provided entity_property_info() informations for LS known fields.
   *
   * For token : Handled tid, completed, firstname, etc... fields and for answer
   * handled id, lastpage, startlanguage, etc.. fields.
   *
   * @param $field
   *   A string, the LS field name.
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array, the entity_property_info() field or FALSE if it is not a fixed
   *   field.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entityPropertyInfo_ls_fields_known($field, $sid) {
    return FALSE;
  }

  /**
   * Provided entity_property_info() informations for LS variable fields.
   *
   * For token : Handled attribute_1, attribute_2, etc... fields and for answer
   * handled 79447X7X1731, 79447X7X1732, etc.. fields.
   *
   * @param $field
   *   A string, the LS field name.
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array, the entity_property_info() field or FALSE if it is not an
   *   existing variable field.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entityPropertyInfo_ls_fields_variables($field, $sid) {
    return FALSE;
  }

  /**
   * Provided entity_property_info() informations for LS fields.
   *
   * Merge known and variable LS fields entity_property_info, returning all LS
   * fields entity_property_info. If a LS field exists on LS but is not defined
   * on Drupal, provide a watchdog information about it.
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the exhaustive LS entity_property_info() fields or an empty
   *   array if there is no LS fields.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entityPropertyInfo_ls_fields($sid = NULL) {

    $lss_entity_type = $this->info['customizable entities']['lss_entity_type'];
    if (empty($sid)) {
      return array();
    }

    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = limesurvey_sync_ls_api_retrieve_ls_fields($lss_entity_type, $sid);
    $ls_current_fields = $existing_ls_fields;
    $fields_entity_property_info = array();
    foreach ($ls_current_fields as $key => $current_ls_field) {
      $field_entity_property_info = $this->entityPropertyInfo_ls_fields_known($current_ls_field, $sid);
      if ($field_entity_property_info) {
        $fields_entity_property_info[$current_ls_field] = $field_entity_property_info;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      foreach ($remaining_fields as $key => $variable_ls_field) {
        $field_entity_property_info = $this->entityPropertyInfo_ls_fields_variables($variable_ls_field, $sid);
        if ($field_entity_property_info) {
          $fields_entity_property_info[$variable_ls_field] = $field_entity_property_info;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !$this->info['base table']) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $sid,
        '%type' => $lss_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey fields %fields have no defined properties into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey fields %fields have no defined properties into Drupal (from %type %sid).', $arg), 'warning');
    }

    // Add ls_sid field.
    $ls_sid_field = array(
      'ls_sid' => array(
        'label' => t('_ID, LimeSurvey survey ID'),
        'description' => t('LimeSurvey survey ID into the LimeSurvey database.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'entity views field' => TRUE,
      ),
    );
    $fields_entity_property_info += $ls_sid_field;
    return (!empty($fields_entity_property_info)) ? $fields_entity_property_info : array();
  }
}

class LimesurveySyncExtraFieldsController extends CustomizableEntitiesExtraFieldsController {
  public function __construct($type) {
    parent::__construct($type);
    // Disable some fields for edition
    $this->disable_extrafields[$this->entityType]['form'][] = 'lss_sync';
    $this->disable_extrafields[$this->entityType]['form'][] = 'lss_status';
    $this->disable_extrafields[$this->entityType]['form'][] = 'lss_last_sync';
    $this->disable_extrafields[$this->entityType]['form'][] = 'lss_last_way';
    if (!empty($this->entityInfo['entity keys']['ls key'])) {
      $this->disable_extrafields[$this->entityType]['form'][] = $this->entityInfo['entity keys']['ls key'];
    }
  }
}

class LimesurveySyncSubentities extends CustomizableEntitiesEntityInfoSubentities {
  /**
   * answers and tokens needs ls_sid from survey.
   */
  protected $source_class = 'LimesurveySyncEntityInfoSurvey';
}

class LimesurveySyncSubentitiesEntityInfo extends LimesurveySyncEntityInfo {

  protected $ls_sid = NULL;
  protected $ls_sid_valid = FALSE;
  protected $title_col = FALSE;

  public function __construct($mymodule = NULL, $source = NULL, $parent_class = NULL, $source_class = NULL) {
    parent::__construct($mymodule, $source, $parent_class, $source_class);
  }

  /*
   * Populate class config datas from ls_sid.
   */
  protected function setup($bundle) {
    // We populate the entity_info parameters from the survey entity.
    $this->ls_sid = $bundle->ls_sid;
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $this->ls_sid_valid = limesurvey_sync_ls_api_ls_table_exists($this->lss_entity_type, $this->ls_sid) || db_table_exists(limesurvey_sync_api_tablename($this->lss_entity_type, $this->ls_sid));
    if ($this->ls_sid_valid) {
      $this->entityname = limesurvey_sync_api_entityname($this->lss_entity_type, $this->ls_sid);
      $this->tablename = limesurvey_sync_api_tablename($this->lss_entity_type, $this->ls_sid);
      $this->tablename_revision = limesurvey_sync_api_tablename($this->lss_entity_type, $this->ls_sid, TRUE);
      $this->customizable_entities_entry['ls_sid'] = $this->ls_sid;
      $sid_suffix = ' ' . $this->ls_sid;
      $this->label = $this->lss_entity_type . $sid_suffix;
      $this->plural_label = $this->lss_entity_type . 's' . $sid_suffix;
      // Rewrite admin_ui, path.
      $typename = limesurvey_sync_api_typename($this->lss_entity_type, $this->ls_sid);
      $typename_url = str_replace('_', '-', $typename);
      $this->crud_path = $typename_url;
      // Use wildcard.
      $survey_type = $bundle->name;
      $survey_type_url = str_replace('_', '-', $survey_type); //'%limesurvey_sync_survey'
      $this->root_admin_ui = "admin/structure/limesurvey/{$this->lss_entity_type}/{$survey_type_url}"; //"admin/structure/limesurvey/surveys/manage/{$survey_type_url}/{$typename_url}";
      $this->overview_path = $this->root_admin_ui;
      $this->entity_label_callback = 'limesurvey_sync_api_default_label';
      $this->revision = TRUE;
      $this->create = TRUE; // FALSE
      $this->update = TRUE; // FALSE
      $this->view = TRUE;
      $this->delete = TRUE;
      //$this->fieldable = FALSE;
    }
    else {
      //throw new Exception('Could not create the LimeSurvey @lss_entity_type entity for @ls_sid survey.', array('@lss_entity_type' => $this->lss_entity_type, '@ls_sid' => $this->ls_sid));
      $this->entityname = '';
    }
  }
}

/**
 * Class managing entities.
 *
 * Extend the common class 'Entity' for entities.
 */
class LimesurveySyncEntity extends CustomizableEntitiesEntity {
  public $lss_entity_type = NULL;

  /**
   * Creates a new entity.
   *
   * Retreive the LS survey ID from the entity type name, making sure to find it
   * for answers and token entities.
   *
   * @see entity_create()
   */
  public function __construct(array $values = array(), $entityType = NULL) {

    parent::__construct($values, $entityType);
    $with_suffix = array('token', 'answer');
    if (in_array($this->lss_entity_type, $with_suffix)) {
      $ls_sid = (!empty($this->entityInfo['customizable entities']['ls_sid'])) ? $this->entityInfo['customizable entities']['ls_sid'] : NULL;
      if (empty($ls_sid)) {
        throw new Exception('Cannot create an instance of ' . $this->lss_entity_type . ' entity without a specified survey id.');
      }
      else {
        $this->ls_sid = $ls_sid;
      }
    }
  }

  /**
   * Returns whether the survey type is locked, thus may not be deleted or renamed.
   *
   */
  public function isLocked() {

    return isset($this->status) && empty($this->is_new) && (($this->status & ENTITY_IN_CODE) || ($this->status & ENTITY_FIXED));
  }
}

/**
 *  A controller implementing exportables stored in the database.
 *
 * Extend the common class 'EntityAPIControllerExportable' for entities.
 */
class LimesurveySyncController extends CustomizableEntitiesEntityController {
  public $lss_entity_type = '';
  public $ls_sid = '';
  public $ls_key = '';

  /**
   * Overridden.
   *
   * Retreive the LS survey ID from the entity type name, making sure to find it
   * for answers and token entities.
   */
  public function __construct($entityType) {

    parent::__construct($entityType);
    $with_suffix = array('token', 'answer');
    $ls_sid = (!empty($this->entityInfo['customizable entities']['ls_sid'])) ? $this->entityInfo['customizable entities']['ls_sid'] : NULL;
    // OR : $this->ls_sid = (!empty($this->entityInfo['customizable entities']['ls_sid'])) ? $this->entityInfo['customizable entities']['ls_sid'] : NULL;
    if (empty($ls_sid) && in_array($this->lss_entity_type, $with_suffix)) {
      throw new Exception('Cannot create an instance of ' . $this->lss_entity_type . ' entity without a specified survey id.');
    }
    elseif (in_array($this->lss_entity_type, $with_suffix)) {
      $this->ls_sid = $ls_sid;
    }
  }

  /**
   * Provide default values on entity custom fields.
   *
   * Each class should implement it.
   *
   * @return
   *   An array, the defaut values keyed by there field name.
   */
  protected function entity_default_values_custom($values = array()) {
    $default_values = parent::entity_default_values_custom();
    $default_values['ls_sync'] = LIMESURVEY_SYNC_UNSYNCHRONIZED;
    $default_values['ls_last_sync'] = 0;
    $dp_default = $this->create_dp_fields_default_values();
    $ls_default = $this->create_ls_fields_default_values();

    return array_merge($default_values, $dp_default, $ls_default);
  }

  /**
   * Provide default values on creating entity.
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the defaut values keyed by there field name.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_dp_fields_default_values($ls_sid = NULL) {

    return array();
  }

  /**
   * Provide default values on creating entity for LS fields.
   *
   * Merge known and variable LS fields default values. If a LS field exists on
   * LS but is not defined on Drupal, provide a watchdog information about it.
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the exhaustive defaut values keyed by there field name for LS
   *   fields.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values($sid = NULL) {

    if (!in_array($this->lss_entity_type, array(
      'token',
      'answer',
    ))) {
      return array();
    }
    $sid = $this->entityInfo['customizable entities']['ls_sid'];
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = limesurvey_sync_ls_api_retrieve_ls_fields($this->lss_entity_type, $sid);
    $ls_current_fields = $existing_ls_fields;
    $default_values = array();
    foreach ($ls_current_fields as $key => $current_ls_field) {
      $field_default_values = $this->create_ls_fields_default_values_known($current_ls_field, $sid);
      if ($field_default_values !== FALSE) {
        $default_values[$current_ls_field] = $field_default_values;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      foreach ($remaining_fields as $key => $variable_ls_field) {
        $field_default_values = $this->create_ls_fields_default_values_variables($variable_ls_field, $sid);
        if ($field_default_values !== FALSE) {
          $default_values[$variable_ls_field] = $field_default_values;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !db_table_exists(limesurvey_sync_api_tablename($this->lss_entity_type, $sid))) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $sid,
        '%type' => $this->lss_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey fields %fields have no default value into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey fields %fields have no default value into Drupal (from %type %sid).', $arg), 'warning');
    }

    return (!empty($default_values)) ? $default_values : array();
  }

  /**
   * Return the corresponding Drupal default value of a LS known field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle common LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic (known)
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_known($field, $sid = NULL) {
    return FALSE;
  }

  /**
   * Return the corresponding Drupal default value of a LS variable field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle variable LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_variables($field, $sid = NULL) {
    return FALSE;
  }

  /**
   * Overridden.
   *
   * Add dynamic title for answers and token entities.
   */
  public function load($ids = array(), $conditions = array()) {

    $entities = parent::load($ids, $conditions);
    if (empty($this->entityInfo['entity keys']['label']) && !empty($this->entityInfo['entity keys']['ls key'])) {
      // For answers and token entities.
      $ls_key = $this->entityInfo['entity keys']['ls key'];
      foreach ($entities as $entity) {
        // Add dynamic title.
        if (!isset($entity->title)) {
          $entity->title = limesurvey_sync_api_default_label($entity, $this->entityType);
        }
        if (!isset($entity->lss_survey)) {
          $entity->lss_survey = limesurvey_sync_api_get_lss_survey($entity, array(), '', $this->entityType);
        }


        // Provide uid from the response table.
        // Note : uid may be an array if various values exists. Pick the first one corresponding to the most recent
        //if (!isset($entity->uid)) {
//  $uid = limesurvey_sync_api_retreive_uid_from_ref_entity($entity);
//  $entity->uid = (!is_array($uid)) ? $uid : current($uid);
        //}
      }
    }
    return $entities;
  }

  /**
   * Permanently saves the given entity, synchronizing it.
   *
   * On saving entity, update/add informations from LimeSurvey to Drupal. Reset
   * the flag 'need_save_ls2dp' to FALSE. Manage date user, log fields. In case
   * of failures, an exception is thrown.
   *
   * @param $entity
   *   The entity to save.
   *
   * @return
   *   SAVED_NEW or SAVED_UPDATED is returned depending on the operation
   *   performed.
   */
  public function save($entity, DatabaseTransaction $transaction = NULL) {

    // Entity can be saved using entity_save(). In order to avoid calling it
    // twice and to synchronize datas on saving, add a flag.
    // Revisions should not be synchronized.
    $is_default_revision = entity_revision_is_default($this->entityType, $entity);
    if ($is_default_revision) {
      // Always reset dp fields on saving.
      $dp_fields = limessurvey_sync_api_update_dp_sync_fields($this->entityType, $entity);
      limesurvey_sync_api_apply_changes($entity, $dp_fields);
    }
    // Use entity_revision_set_default($entity_type, $entity) for make-current.
    parent::save($entity, $transaction);
  }

  /**
   * Builds a structured array representing the entity's content.
   *
   * The content built for the entity will vary depending on the $view_mode
   * parameter.
   *
   * @param $entity
   *   An entity object.
   * @param $view_mode
   *   View mode, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   *
   * @return
   *   The renderable array.
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {

    // Our additions to the $build render array
    // $wrapper = entity_metadata_wrapper($this->entityType, $entity);
    if (!in_array($view_mode, array('admin_dp', 'admin_ls'))) {
      $fieldsets = drupal_get_form('limesurvey_sync_api_admin_block', $entity, $this->entityType, $langcode);
      $content['admin'] = $fieldsets;
      $content['admin']['#weight'] = -100;
    }
    $build = parent::buildContent($entity, $view_mode, $langcode, $content);

    return $build;
  }

  protected function renderEntityProperty($wrapper, $name, $property, $view_mode, $langcode, &$content) {
    parent::renderEntityProperty($wrapper, $name, $property, $view_mode, $langcode, $content);
    if (isset($content[$name]) && drupal_substr($name, 0, 3) == 'ls_') {
      $funct = 'limesurvey_sync_' . $this->lss_entity_type . '_display_field';
      if (function_exists($funct)) {
        if ($value = call_user_func($funct, $name, $property->value(), 'value', $this->ls_sid)) {
          $content[$name]['#content'] = $value;
        }
      }
    }
    elseif ($name == 'lss_sync') {
      $css = array(
        'css' => array(
          drupal_get_path('module', 'limesurvey_sync') . '/misc/limesurvey_sync.css',
        ),
      );
      // Check sync status
      $entity = $wrapper->value();
      $synchronize_set = limesurvey_sync_synchronize($wrapper->type(), $entity);
      $entity_id = $wrapper->{$this->entityInfo['entity keys']['id']}->value();
      $need_sync = FALSE;
      $need_sync_entity = (!empty($synchronize_set['need_sync']) && array_key_exists($entity_id, $synchronize_set['need_sync']));
      $need_sync_ref = (!empty($synchronize_set['need_sync_ref']) && array_key_exists($entity_id, $synchronize_set['need_sync_ref']));
      $lss_sync = $wrapper->lss_sync->value();
      if ($need_sync_entity || $need_sync_ref) {
        $need_sync = TRUE;
        // Build the synchronized entity.
        if ($need_sync_entity) {
          //$sync_entity = $synchronize_set['need_sync'][$entity->$dp_key]['dp'];
          if (isset($synchronize_set['need_sync'][$entity_id]['diff']['lss_sync'])) {
            $lss_sync = $synchronize_set['need_sync'][$entity_id]['diff']['lss_sync'];
          }
        }
      }
      $content[$name] = array(
        'data' => array(
          '#theme' => 'limesurvey_sync_field_label',
          '#field' => 'lss_sync',
          '#to_display' => $lss_sync,
          '#lss_entity_type' => $wrapper->lss_entity_type->value(),
          '#ls_sid' => $wrapper->ls_sid->value(),
          '#need_sync' => $need_sync,
          '#attached' => $css,
        ),
      );
    }
  }

  /**
   * Overridden.
   *
   * Delete associated preconfigured rules.
   */
  public function delete($ids, DatabaseTransaction $transaction = NULL) {
    $rulesnames = array();
    $entities = $ids ? $this->load($ids) : FALSE;
    foreach ($entities as $id => $entity) {
      if ($this->lss_entity_type == 'survey') {
        $ls_sid = $entity->ls_sid;
        foreach (array('answer', 'token') as $lss_entity_type) {
          $entity_type = limesurvey_sync_api_entityname($lss_entity_type, $ls_sid);
          if ($entity_type && $info = entity_get_info($entity_type)) {
            // The token or answer entity exists and is going to be deleted.
            foreach (array('ls2dp', 'dp2ls') as $way) {
              $rulesnames[] = limesurvey_sync_api_rules_names($entity_type, 'sync', $way);
            }

            if ($lss_entity_type == 'token') {
              $components = array('import_user_mail', 'import_user_name', 'create_user');
              foreach ($components as $component) {
                $way = ($component == 'create_user') ? 'dp2ls' : 'ls2dp';
                $rulesnames[] = limesurvey_sync_api_rules_names($entity_type, $component, $way);
              }

              foreach (array('ls2dp', 'dp2ls') as $way) {
                $rulesnames[] = limesurvey_sync_api_rules_names($entity_type, 'import_user_datas', $way);
              }
            }
          }
        }
      }
    }

    parent::delete($ids, $transaction);

    if (!empty($rulesnames)) {
      $rulesid_to_del = array();
      foreach ($rulesnames as $rulesname) {
        if ($rule = entity_load_single('rules_config', $rulesname)) {
          $rulesid_to_del[] = $rule->id;
        }
      }
      if (!empty($rulesid_to_del)) {
        rules_config_delete($rulesid_to_del);
      }
    }
  }
}

/**
 * Default controller for providing UI.
 *
 * Extend the common class 'EntityDefaultUIController' for entities.
 */
class LimesurveySyncUIController extends CustomizableEntitiesEntityUIController {
  // Edit the menu from EntityDefaultUIController::hook_menu().
  public $lss_entity_type = '';
  public $ls_sid = '';
  protected $menu_weight = 0;
  protected $ls_sub_entity = FALSE;
  protected $hide_export_status = TRUE;

  /**
   * Overridden.
   *
   * Retreive the LS survey ID from the entity type name, making sure to find it
   * for answers and token entities.
   */
  public function __construct($entity_type, $entity_info) {

    parent::__construct($entity_type, $entity_info);
    $with_suffix = array('token', 'answer');
    $ls_sid = (!empty($this->entityInfo['customizable entities']['ls_sid'])) ? $this->entityInfo['customizable entities']['ls_sid'] : NULL;
    if (empty($ls_sid) && in_array($this->lss_entity_type, $with_suffix)) {
      throw new Exception('Cannot create an instance of ' . $this->lss_entity_type . ' entity without a specified survey id.');
    }
    else {
      $this->ls_sid = $ls_sid;
    }
  }

  /**
   * Provides definitions for implementing hook_menu().
   *
   * Manage entity view page, adding, synchronize (one and all) pages, revisions,
   * import, clone, devel pages.
   */

  public function hook_menu() {
    $items = array();
    if ($this->ls_sub_entity) {
      $items["admin/structure/limesurvey/{$this->lss_entity_type}"] = array(
        'access callback' => array(TRUE),
        //'type' => MENU_NORMAL_ITEM,
        'title' => drupal_strtoupper(drupal_substr($this->lss_entity_type, 0, 1)) . drupal_substr($this->lss_entity_type, 1),
        'weight' => $this->menu_weight,
      );

    }
    // In order to works with the menu router, the modules must be called in
    // this order : entity, limesurvey_sync, limesurvey_sync_survey, the answer
    // and token, depending on how the hook_entity_info() functions are called.
    $items += parent::hook_menu();
    // Ad specific stuf for sub entities.
    if ($this->ls_sub_entity) {
      // Set token and answer list as tab instead of callback.
      $overview_path = $this->entityInfo['customizable entities']['overview path'];
      if (isset($items[$overview_path])) {
        $items[$overview_path]['type'] = MENU_NORMAL_ITEM; //MENU_LOCAL_TASK;
        $items[$overview_path]['weight'] = $this->menu_weight;
        //$items[$overview_path]['access callback'] = 'limesurvey_sync_api_subentity_tab_access';
        //$items[$overview_path]['access arguments'] = array('update', $this->entityType, (count(explode('/', $this->path)) -2));
      }
    }
    // Cloning a entry may be a problem for linking drupal and limesurvey.
    // Disallow it !
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%entity_object';
    $path_view = "{$this->entityInfo['customizable entities']['crud path']}/{$wildcard}";
    $path_clone = $path_view . '/clone';
    if (isset($items[$path_clone])) {
      // I keep it untill I 'm sure it's a trouble.
      //unset($items[$path_clone]);
    }
    if ($this->lss_entity_type ==  'survey') {
      // Adapt the survey view page for the menu : not menu callback.
      $items[$path_view]['type'] = MENU_NORMAL_ITEM;
    }

    // Add synchronize link.
    $id_count = count(explode('/', $path_view));
    $placeholder_view = $id_count - 1;

    // Set a synchronization page for all entities.
    $items[$this->entityInfo['customizable entities']['overview path'] . '/synchronize-all'] = array(
      'title' => 'Synchronize all ' . $this->lss_entity_type,
      'page callback' => 'drupal_get_form',
      'page arguments' => array(
        'limesurvey_sync_api_synchronize_page',
        $this->entityType,
        FALSE,
      ),
      'access arguments' => array(customizable_entities_permission_item('administer', limesurvey_sync_api_entityname($this->lss_entity_type))),
      'weight' => -5,
      'type' => MENU_LOCAL_ACTION,
      'file' => 'limesurvey_sync_api.pages.inc',
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
    );

    // Set a synchronization page for 1 entity.
    $items[$path_view . '/synchronize'] = array(
      'title' => 'Synchronize',
      // 'title callback' => 'entity_label',
      // 'title arguments' => array($this->entityType, $id_count + 1),
      'page callback' => 'drupal_get_form',
      'page arguments' => array(
        'limesurvey_sync_api_synchronize_page',
        $this->entityType,
        $placeholder_view,
      ),
      'load arguments' => array($this->entityType),
      'access callback' => 'entity_access',
      'access arguments' => array(
        'update',
        $this->entityType,
        $placeholder_view,
      ),
      'type' => MENU_LOCAL_TASK,
      'file' => 'limesurvey_sync_api.pages.inc',
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
    );

    // Mofifie revert revision callback
    if (isset($items[$path_view . '/revisions/%/revert'])) {
      $items[$path_view . '/revisions/%/revert']['page arguments'][0] = 'limesurvey_sync_api_revision_revert_confirm';
      $items[$path_view . '/revisions/%/revert']['file'] = 'limesurvey_sync_api.pages.inc';
    }

    return $items;
  }

  /**
   * Provides definitions for implementing hook_forms().
   *
   * Use per bundle form ids if possible, such that easy per bundle alterations
   * are supported too.
   *
   * Manage editing form for answers and tokens entities. Note that for
   * performance reasons, this method is only invoked for forms, which receive
   * the entity_type as first argument. Thus any forms added, must follow that
   * pattern.
   *
   * @see entity_forms()
   */
  public function hook_forms() {

    $forms = parent::hook_forms();
    if ($this->ls_sub_entity) {
      foreach (limesurvey_sync_survey_get_surveys() as $type) {
        $entity_type = limesurvey_sync_api_entityname($this->lss_entity_type, $type->ls_sid);
        $form_id_value = $entity_type . '_form';
        $forms[$form_id_value] = array(
          'callback' => "limesurvey_sync_{$this->lss_entity_type}_form",
          //'wrapper_callback' => 'entity_ui_form_defaults',
        );
      }
    }

    return $forms;
  }
}

function limessurvey_sync_api_update_dp_sync_fields($entity_type, $entity) {
  $is_default_revision = entity_revision_is_default($entity_type, $entity);
  if (!$is_default_revision) {
    return array();
  }
  $lss_status_funct = "limesurvey_sync_api_{$entity->lss_entity_type}_lss_status";
  $dp_fields = array();
  if (function_exists($lss_status_funct)) {
    $dp_fields['lss_status'] = call_user_func($lss_status_funct, $entity_type, $entity);
  }
  $dp_fields['lss_sync'] = limesurvey_sync_api_lss_sync($entity_type, $entity);

  if (!empty($entity->lss_entity_type) && $entity->lss_entity_type == 'token') {
    // Set the owner always according to the ls email value.
    if (!empty($entity->ls_email) && $account_ls_mail = user_load_by_mail($entity->ls_email)) {
      $dp_fields['lss_uid'] = $account_ls_mail->uid;
    }

    if (!empty($entity->need_save_dp2ls) && empty($entity->ls_token)) {
      // create unique token value
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_token');
      $dp_fields['ls_token'] = limesurvey_sync_ls_api_token_create_unique_token($entity->ls_sid);
      // ls_email from lss_uid will be set from Rules.
    }
  }
  elseif (!empty($entity->lss_entity_type) && $entity->lss_entity_type == 'answer') {
    $token_fieldname = limesurvey_sync_api_et_fieldname('token', $entity->ls_sid);
    $token_entityname = limesurvey_sync_api_entityname('token', $entity->ls_sid);
    // Autoset the token field.
    if (!empty($entity->ls_token) && limesurvey_sync_api_answer_tokenpersistance($entity->ls_sid)) {
      $dp_tid = (isset($entity->{$token_fieldname}) && !empty($entity->{$token_fieldname}[LANGUAGE_NONE][0]['target_id'])) ? $entity->{$token_fieldname}[LANGUAGE_NONE][0]['target_id'] : FALSE;
      $token_info = entity_get_info($token_entityname);
      $token = entity_load_single($token_entityname, $dp_tid);
      if (!$dp_tid || !$token) {
        // The token entity field is not correctly set.
        // First retreive the token entity from token value.
        $token_id_field = $token_info['entity keys']['id'];
        $dp_tid = db_query('SELECT ' . $token_id_field . ' FROM {' . $token_info['base table'] . '} WHERE ls_token = :ls_token ORDER BY changed DESC', array(':ls_token' => $entity->ls_token))->fetchField();
        if (!empty($dp_tid)) {
          // The corresponding token entity exists on drupal.
          $token = entity_load_single($token_entityname, $dp_tid);
        }
        else {
          // Retreive the LS token.
          module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
          if ($row_token = limesurvey_sync_ls_api_retrieve_ls_row_from_token($entity->ls_sid, $entity->ls_token, 'token')) {
            // The token exists on ls. Import it on drupal.
            $token_ls_id_field = $token_info['entity keys']['ls key'];
            $token = entity_create($token_entityname, array($token_ls_id_field => $row_token[$token_ls_id_field]));
            $token->need_save_ls2dp = TRUE;
            entity_save($token_entityname, $token);
            $dp_tid = $token->$token_id_field;
          }
        }
        if ($dp_tid) {
          // Export the entity reference value to be added to the answer entity.
          $dp_fields[$token_fieldname][LANGUAGE_NONE][0]['target_id'] = $dp_tid;
        }
      }

      // Set answer author according to the token lss_uid (the user this token
      // is for).
      if ($token) {
        if (!empty($token->lss_uid)) {
          // export the answer entity author to be added to the answer entity.
          $dp_fields['uid'] = $token->lss_uid;
        }
      }
    }


    if (!empty($entity->lss_last_sync) && $entity->lss_last_sync == REQUEST_TIME) {
      // The answer has just been synchronized.
      // Make sure the token is synchronized.
      if ($token) {
        $synchronize_set = limesurvey_sync_synchronize($token_entityname, $token);
        if (!empty($synchronize_set['need_sync'])) {
          $token->need_save_ls2dp = TRUE;
          entity_save($token_entityname, $token);
        }
      }
    }
  }
  return $dp_fields;
}

/**
 * Test the synchronization from LimeSurvey to Drupal or Dp2LS..
 *
 * Compare the DP entry to the current LS one. Update the entity adding a flag
 * 'need_save_ls2dp' to TRUE or FALSE. This function can be called for one
 * entity or for all entities from this entity type.
 *
 * @param $entity
 *   The entity object to save or 'all' for all entities from this entity
 *   type, or 'all_skip_import' for all entities from this entity
 *   type but without returning (for performance raisons) entities that
 *   have never been saved on DP.
 * @param $way
 *   A string, the synchronization way. 'ls2dp' or 'dp2ls'. Default is 'ls2dp'.
 *
 * @return
 *   An array. Keys are ['ok'][dp_primary_key_value] if synchronization is
 *   not needed; ['need_sync'] if synchronization is needed, with subkeys :
 *   ['need_sync'][dp_primary_key_value]['dp'] the DP current entity object,
 *   ['need_sync'][dp_primary_key_value]['diff'] an array of the changes keyed
 *   by the properties; ['need_sync'][dp_primary_key_value]['dp_date'] the
 *   last changed date on DP and ['need_sync'][dp_primary_key_value]['ls_date']
 *   the last changed date on LS (used to compare witch one is the most
 *   recent, for selecting ls2dp or dp2ls synchronization in future
 *   developpements). At last ['need_import'][ls_primary_key_value] as the
 *   converted to DP entity object to save for LS entries that have no entry
 *   in DP. ['need_import'] only exists with the 'all' parameter. Note : for
 *   entity to create, the dp_primary_key_value from
 *   ['need_sync'][dp_primary_key_value] is 'this'. It should be taken in
 *   consideration when browsing the result of the ls_synchronize()
 *   method.
 */
function limesurvey_sync_synchronize($entity_type, $entity, $way = 'ls2dp') {

  $info = entity_get_info($entity_type);

  $dp_key = $info['entity keys']['id'];
  $ls_key = isset($info['entity keys']['ls key']) ? $info['entity keys']['ls key'] : '';
  $this_ls_id = FALSE;
  $this_dp_id = FALSE;
  $lss_entity_type = isset($info['customizable entities']['lss_entity_type']) ? $info['customizable entities']['lss_entity_type'] : '';
  $ls_sid = isset($info['customizable entities']['ls_sid']) ? $info['customizable entities']['ls_sid'] : '';

  // Retrieve from Drupal.
  if ($entity != 'all' && $entity != 'all_skip_import') {
    // If an antity is provided sync only this one.
    // The entity may have been altered : use this one.
    $entities = array($entity);
    // Only one entity that have not beeing created yet. Use 'this' index.
    $this_dp_id =  (!empty($entity->$dp_key)) ? $entity->$dp_key : 'this';

    // For retreiving from LS.
    if ($ls_key && !empty($entity->$ls_key)) {
      $this_ls_id = $entity->$ls_key;
    }
  }
  else {
    // Sync all entities.
    // Retrieve from Drupal.
    $entities = entity_load($entity_type);
    ksort($entities);
  }

  $synchronize_set = array();
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
  // Retrieve form LS.
  $selection = array();
  if (in_array($lss_entity_type, array('answer', 'token'))) {
    if ($entity != 'all' && $entity != 'all_skip_import') {
      $selection = $entities;
    }
  }
  $ls_ids = limesurvey_sync_api_existing_ls_ids($entity_type, $selection);

  $synchronize_set['need_import'] = $ls_ids;

  $lss_status_funct = "limesurvey_sync_api_{$lss_entity_type}_lss_status";
  if (!function_exists($lss_status_funct)) {
    $lss_status_funct = FALSE;
  }

  foreach ($entities as $record) {
    $dp_key_value = ($this_dp_id != 'this') ? $record->$dp_key : 'this';
    // Break the association object. Works on array.
    $dp_row = array();
    foreach ($record as $key => $value) {
      $dp_row[$key] = $value;
    }

    $ls_row_converted = array('lss_sync' => limesurvey_sync_api_lss_sync($entity_type, $record, $ls_ids));
    if ($lss_status_funct) {
      $ls_row_converted['lss_status'] = call_user_func($lss_status_funct, $entity_type, $record);
    }
    // Don't change the values for LS fields if not synchronized.
    if ($ls_row_converted['lss_sync'] == LIMESURVEY_SYNC_SYNCHRONIZED) {
      // This dp entry does exists in both dp and ls : an update may be needed.
      // On the survey entities, only sync and status are concerned.
      if (!in_array($lss_entity_type, array('survey'))) {
        $ls_row = $ls_ids[$record->$ls_key];
        // Convert LS fields values to drupal.
        $ls_row_converted = array_merge($ls_row_converted, limesurvey_sync_ls_api_fields_convert_values($ls_row, $lss_entity_type, $ls_sid, 'ls2dp'));
        // And update status values.
        if ($lss_status_funct) {
          $ls_row_converted['lss_status'] = call_user_func($lss_status_funct, $entity_type, $ls_row_converted);
        }
      }
      // Delete entries that have been handled.
      if ($ls_key) {
        unset($synchronize_set['need_import'][$record->$ls_key]);
      }
    }
    // @TODO : for dp2ls on all entities, look for desynchronized entities,
    // proposing to recreate them on LS.
    // Compare both.
    if ($way == 'ls2dp') {
      $diff = array_diff_assoc($ls_row_converted, $dp_row);
    }
    else {
      $ls_fields = array_flip(limesurvey_sync_ls_api_retrieve_ls_fields($lss_entity_type, $ls_sid));
      // Select the LS fields.
      $dp_row_ls_fields = array_intersect_key($dp_row, $ls_fields);
      $diff = array_diff_assoc($dp_row_ls_fields, $ls_row_converted);
    }
    if (!empty($diff)) {
      // The entity need to be resync and save !
      $dp_row_sync = $dp_row;
      foreach ($diff as $field => $value) {
        $dp_row_sync[$field] = $value;
      }
      if (!empty($info['entity keys']['revision'])) {
        $dp_row_sync['log'] = format_plural(count($diff), 'Change on the field %fields', 'Change on the fields : %fields', array(
          '%fields' => implode(', ', array_keys($diff)),
        ));
        $dp_row_sync['revision'] = TRUE;
      }

      // Save the result into an object. Use entity_save() on it to save it.
      $synchronize_set['need_sync'][$dp_key_value]['dp'] = $record;
      $synchronize_set['need_sync'][$dp_key_value]['diff'] = $diff;
      // Save Dates, it may have an interest for saving the most recent one.
      if (!empty($info['entity keys']['ls date key']) && !empty($dp_row_sync['changed']) && isset($ls_row_converted[$info['entity keys']['ls date key']])) {
        // No date for survey.
        $synchronize_set['need_sync'][$dp_key_value]['dp_date'] = $dp_row_sync['changed'];
        $synchronize_set['need_sync'][$dp_key_value]['ls_date'] = $ls_row_converted[$info['entity keys']['ls date key']];
      }
      // if it match the entity requested : update it with the new values.
      if ($entity != 'all' && $this_dp_id && $this_dp_id == $dp_key_value) {
        // $entity is provided by reference. update it with a flag.
        $entity->{"need_save_$way"} = TRUE;
      }
    }
    else {
      // If no change, do not store them for performance raisons.
      $synchronize_set['ok'][$dp_key_value] = TRUE;
      // if it match the entity requested : set a no sync flag..
      if ($entity != 'all' && $this_dp_id && $this_dp_id == $dp_key_value) {
        // $entity is provided by reference. Update it.
        //$entity->{"need_save_$way"} = FALSE;
      }
    }
  }

  // All entities from Drupal have been explored. Look for remaining entities
  // to import from LS.
  // Don't try to import survey : it only manually.
  if ($way == 'ls2dp' && !empty($synchronize_set['need_import']) && $lss_entity_type != 'survey') {
    foreach ($synchronize_set['need_import'] as $ls_id => $ls_row) {
      $ls_row_converted = limesurvey_sync_ls_api_fields_convert_values($ls_row, $lss_entity_type, $ls_sid, 'ls2dp');
      $ls_row_converted['lss_sync'] = LIMESURVEY_SYNC_SYNCHRONIZED;
      if ($lss_status_funct) {
        $ls_row_converted['lss_status'] = call_user_func($lss_status_funct, $entity_type, $ls_row_converted);
      }
      $entity_to_import = entity_create($entity_type, $ls_row_converted);
      // Add a flag to not synchronize twice from the save() method.
      $entity_to_import->need_save_ls2dp = TRUE;
      // Remplace into the synchronize_set et LS array by the new entity
      // object. It can be created using entity_save().
      $synchronize_set['need_import'][$ls_id] = $entity_to_import;
      if ($entity != 'all' && $ls_id == $this_ls_id) {
        // $entity is provided by reference. update it with a flag.
        $entity->need_save_ls2dp = TRUE;
      }
    }
  }

  if (empty($synchronize_set['need_import'])) {
    unset($synchronize_set['need_import']);
  }

  return $synchronize_set;
}


function limesurvey_sync_api_apply_synchronization($entity_type, $entity) {
  $is_default_revision = entity_revision_is_default($entity_type, $entity);
  if (!$is_default_revision) {
    return;
  }

  $way = (isset($entity->need_save_ls2dp) && $entity->need_save_ls2dp) ? 'ls2dp' : ((isset($entity->need_save_dp2ls) && $entity->need_save_dp2ls) ? 'dp2ls' : FALSE);
  if ($way) {
    $synchronize_set = limesurvey_sync_synchronize($entity_type, $entity, $way);
    $info = entity_get_info($entity_type);
    $id_key = $info['entity keys']['id'];
    $dp_key_value = $dp_key_value = (!empty($entity->$id_key)) ? $entity->$id_key : 'this';
    $sync = FALSE;

    if ($way == 'ls2dp') {

      // If synchronization needed : apply the changes.
      if (!empty($synchronize_set['need_sync'])) {
        $sync = TRUE;
        $diff = $synchronize_set['need_sync'][$dp_key_value]['diff'];
        // Alter the entity according to the LS values.
        limesurvey_sync_api_apply_changes($entity, $diff);
        // Reset the flag. No : Rules use it
        //$entity->need_save_ls2dp = FALSE;
        // Save revision on each changes on LS.
        $entity->revision = TRUE;
        // Set a log message expliciting diff fields.
        $entity->log = t('Sync LS -> DP : changes on @fields.', array('@fields' => implode(', ', array_keys($diff))));
      }
    }
    elseif ($way == 'dp2ls') {
      $succeed = FALSE;
      // If synchronization needed : apply the changes.
      $diff = array();
      if (!empty($synchronize_set['need_sync'])) {
        $sync = TRUE;
        $diff = $synchronize_set['need_sync'][$dp_key_value]['diff'];
        // Update the LS entry.
        module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
        // Below : update LS database and set the corresponding ls key to $entity.
        $succeed = limesurvey_sync_ls_api_edit_row($entity_type, $entity);
      }
      if ($succeed) {
        // Reset the flag.
        //$entity->need_save_dp2ls = FALSE;
        // Save revision on each changes on LS.
        $entity->revision = TRUE;
        // Set a log message expliciting diff fields.
        $entity->log = t('Sync DP -> LS : changes on @fields.', array('@fields' => implode(', ', array_keys($diff))));
      }
      else {
        $entity->revision = TRUE;
        // Set a log message expliciting diff fields.
        $entity->log = t('Sync DP -> LS has failed');
      }
    }

    if ($sync) {
      // Handle Sync properties.
      $entity->lss_last_sync = REQUEST_TIME;
      $entity->lss_last_way = $way;
    }
  }


  // Always reset dp fields (lss_sync, lss_status and more as they may have
  // changed from Rules.
  $dp_fields = limessurvey_sync_api_update_dp_sync_fields($entity_type, $entity);
  limesurvey_sync_api_apply_changes($entity, $dp_fields);
}

/**
 * Title for token and answser is not stored int Dabase but dynamically generated.
 *
 * @param $entity
 *   The answer or token entity object.
 * @param $entity_type
 *   The answer or token entity type.
 *
 * @return
 *   A string, the sanitized and tranlated title including the primary ls key
 *   value.
 *
 * @see entity_label()
 */
function limesurvey_sync_api_default_label($entity, $entity_type) {
  $label = '';
  $info = entity_get_info($entity_type);
  if (!empty($info['entity keys']['ls key'])) {
    $ls_key = $info['entity keys']['ls key'];
    if (isset($entity->$ls_key)) {
      $id = $entity->$ls_key;
      $arg = array(
        '@id' => $id,
        '@type' => drupal_ucfirst($entity->lss_entity_type),
        '@sid' => $entity->ls_sid,
      );
      $label = t('@type id #@id from the survey #@sid', $arg);
    }
  }
  return $label;
}

/**
 * Access check for multiple permissions, return TRUE if at least one is TRUE.
 *
 * If return FALSE, the user will not have access to any of the LimeSurvey sync
 * administration menus ! It corresponds to the 'admin/structure/limesurvey'
 * menu root.
 *
 * @param $list
 *   An array of permissions to pass to the user_access() function. If empty,
 *   check all administration permissions on the LimeSurvey_sync module. Default
 *   is empty.
 * @param $account
 *   A user object. If NULL the connected user is used. Default is NULL.
 *
 * @return
 *   A Boolean. TRUE for access.
 */
function limesurvey_sync_api_multiple_access_check($list = array(), $account = NULL) {

  if ($list === array()) {
    // Load all main LimeSurvey module access.
    $list = array(
      customizable_entities_permission_item('administer', limesurvey_sync_api_entityname('survey')),
      customizable_entities_permission_item('administer', limesurvey_sync_api_entityname('answer')),
      customizable_entities_permission_item('administer', limesurvey_sync_api_entityname('token')),
    );
  }
  foreach ($list as $right) {
    if (user_access($right, $account)) {
      return TRUE;
    }
  }
  return FALSE;
}

function limesurvey_sync_api_retreive_response_from_ref_entity($ref_entity) {
  if (empty($ref_entity->ls_sid)) {
    return array();
  }
  $response_entityname = limesurvey_sync_api_entityname('response');
  $ref_entityname = limesurvey_sync_api_entityname($ref_entity->lss_entity_type, $ref_entity->ls_sid);
  $ref_field_name = limesurvey_sync_api_et_fieldname($ref_entity->lss_entity_type, $ref_entity->ls_sid);
  $survey_typename = limesurvey_sync_api_typename('survey', $ref_entity->ls_sid);
  $ref_entity_info = entity_get_info($ref_entityname);
  $dp_key = $ref_entity_info['entity keys']['id'];
  $dp_key_value = $ref_entity->$dp_key;
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', $response_entityname)
  ->entityCondition('bundle', $survey_typename)
  ->fieldCondition($ref_field_name, 'target_id', $dp_key_value, '=' )
  //->range(0, 10)
  ->propertyOrderBy('dp_rid', 'DESC')
  ->addMetaData('account', user_load(1)); // Run the query as user 1.

  $result = $query->execute();
  $all_responses = array();
  if (isset($result[$response_entityname])) {
    $all_responses = entity_load($response_entityname, array_keys($result[$response_entityname]));
  }

  return $all_responses;
}

function limesurvey_sync_api_retreive_uid_from_ref_entity($ref_entity) {
  // @TODO try from the participant entity if token persistance enabled.
  $all_responses = limesurvey_sync_api_retreive_response_from_ref_entity($ref_entity);
  $all_uids = array();
  foreach ($all_responses as $dp_rid => $response_entity) {
    $all_uids[$dp_rid] = $response_entity->uid;
  }
  $unique_uids = array_unique($all_uids);
  if (count($unique_uids) == 1) {
    $uid = current($unique_uids);
  }
  elseif (empty($unique_uids)) {
    $uid = 0; // set as anonymous.
  }
  else {
    // Many uids associated to this reference, return an array.
    $uid = array_values($unique_uids);
  }
  return $uid;
}

/**
 * Return the survey ID from the type name.
 *
 * Example : 'ls_survey_123456' will return '123456'.
 *
 * @param $typename
 *   An string. Corresponding to the variable name.
 * @param $lss_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 *
 * @return
 *   A numeric, the LS survey ID or FALSE if error or NULL if invalid
 *   parametters.
 */
function limesurvey_sync_api_retrieve_ls_sid_from_typename($typename, $lss_entity_type) {

  $entity_types = array('token', 'answer', 'survey');
  if (!empty($typename) && in_array($lss_entity_type, $entity_types)) {
    $ls_sid = str_replace(limesurvey_sync_api_typename($lss_entity_type, 'prefix'), '', $typename);
    if (empty($ls_sid) || !is_numeric($ls_sid)) {
      return FALSE;
    }
    return $ls_sid;
  }
  return NULL;
}

/**
 * Return the LS entity type from the entity type name.
 *
 * Example : 'limesurvey_sync_answer_123456' will return 'answer'.
 *
 * @param $entity_type
 *   The entity type of the entity.
 *
 * @return
 *   A string, the LS entity type or FALSE if error or NULL if invalid
 *   parametters.
 */
function limesurvey_sync_api_retrieve_lss_entity_type_from_entityname($entity_type) {

  if (empty($entity_type) || !is_string($entity_type)) {
    return FALSE;
  }
  $plit = explode('_', $entity_type);
  $count = count($plit);
  if ($count < 3 || strpos($entity_type, 'limesurvey_sync_') !== 0) {
    return FALSE;
  }
  $with_suffix = array('token', 'answer');
  $without_suffix = array('survey', 'response');
  $lss_entity_type = $plit[2];
  if (!in_array($lss_entity_type, array_merge($with_suffix, $without_suffix)) || (in_array($lss_entity_type, $without_suffix) && $count > 3) || (in_array($lss_entity_type, $with_suffix) && ($count > 4 || !is_numeric($plit[3])))) {
    return FALSE;
  }
  return $lss_entity_type;
}

/**
 * Provide the LimeSurvey Sync module entity synchronzation status.
 *
 * Check if the Drupal entity is binded to a valid LimeSurvey entry.
 *
 * @param $entity_type
 *   The entity type.
 * @param $entity
 *   The entity object.
 * @param $existing_ls_enties_keyed_by_ls_key
 *   An array : the LimeSurvey entries keyed by the LS primary key. For
 *   performance raisons, this parameter will usually contain only one key
 *   corresponding of the one from the DP entry. Default is empty.
 *
 * @return
 *   A numeric defined as a constant : LIMESURVEY_SYNC_SYNCHRONIZED (1) for
 *   synchronized, LIMESURVEY_SYNC_DESYNCHRONIZED (2) for desynchronized (this
 *   entry had exist into the ls table but has been deleted into the ls table)
 *   or LIMESURVEY_SYNC_UNSYNCHRONIZED (3) for unsynchronized (this dp entry
 *   has never had a reference into the ls table).
 */
function limesurvey_sync_api_lss_sync($entity_type, $entity, $existing_ls_enties_keyed_by_ls_key = array()) {

  if (empty($existing_ls_enties_keyed_by_ls_key)) {
    $existing_ls_enties_keyed_by_ls_key = limesurvey_sync_api_existing_ls_ids($entity_type, array($entity));
  }
  $info = entity_get_info($entity_type);
  $ls_key = isset($info['entity keys']['ls key']) ? $info['entity keys']['ls key'] : '';
  if (!$ls_key || empty($entity->$ls_key)) {
    // This dp entry has never had a reference into the ls table.
    return LIMESURVEY_SYNC_UNSYNCHRONIZED;
  }
  elseif (array_key_exists($entity->$ls_key, $existing_ls_enties_keyed_by_ls_key)) {
    return LIMESURVEY_SYNC_SYNCHRONIZED;
  }
  else {
    // This entry had exist into the ls table but has been deleted into
    // the ls table.
    return LIMESURVEY_SYNC_DESYNCHRONIZED;
  }
}

/**
 * List of the lss_status possible values.
 *
 * Used for compatibility with Views and Rules modules, throught
 * hook_entity_property_info().
 *
 * @return
 *   An array, the keys are the lss_status value and values are the human
 *   corresponding display message.
 */
function limesurvey_sync_api_lss_sync_status_list() {
  $status = array(
    LIMESURVEY_SYNC_SYNCHRONIZED,
    LIMESURVEY_SYNC_DESYNCHRONIZED,
    LIMESURVEY_SYNC_UNSYNCHRONIZED,
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($status as $value) {
    $list[$value] = limesurvey_sync_api_display_field('survey', 'lss_sync', $value);
  }
  return $list;
}

function limesurvey_sync_api_lss_last_way_list() {
  $options = array(
    '',
    'ls2dp',
    'dp2ls',
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($options as $value) {
    $list[$value] = limesurvey_sync_api_display_field('survey', 'lss_last_way', $value);
  }
  return $list;
}

/**
 * List of the lss_entity_type values.
 *
 * Used for compatibility with Views and Rules modules, throught
 * hook_entity_property_info().
 *
 * @return
 *   An array, the keys are the lss_entity_type value and values are the human
 *   corresponding display message.
 */
function limesurvey_sync_api_lss_entity_type_list() {
  return array(
    'survey' => t('Survey'),
    'response' => t('Response'),
    'answer' => t('Answer'),
    'token' => t('Token'),
  );
}

/**
 * Return a type name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'ls_answer_123456' for 'answer' and '123456'.
 *
 * @param $lss_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID or the string 'prefix' for returning the prefix
 *   ('ls_answer_' in the example). Or FALSE if an error occures.
 *
 * @return
 *   An entity object.
 */
function limesurvey_sync_api_typename($lss_entity_type, $ls_sid = FALSE) {

  if (empty($ls_sid)) {
    return FALSE;
  }
  $prefix = "ls_{$lss_entity_type}_";
  if ($ls_sid == 'prefix') {
    $ls_sid = '';
  }
  switch ($lss_entity_type) {
    case 'prefix':
    case 'survey':
    case 'answer':
    case 'token':
    case 'response':
      // Return 'ls_survey_123456' or 'ls_answer_123456', or 'ls_token_123456'.
      return "ls_{$lss_entity_type}_{$ls_sid}";
  }
  return FALSE;
}

/**
 * Return field name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'field_ls_answer_123456' for 'answer' and '123456'. It
 * is used by the Field core module.
 *
 * @param $lss_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is FALSE.
 *
 * @return
 *   Astring, the field name.
 */
function limesurvey_sync_api_et_fieldname($lss_entity_type, $ls_sid = FALSE) {
  return 'field_' . limesurvey_sync_api_typename($lss_entity_type, $ls_sid);
}

/**
 * Return entity name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'limesurvey_sync_answer_123456' for 'answer' and
 * '123456'.
 *
 * @param $lss_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is FALSE.
 *
 * @return
 *   Astring, the field name.
 */
function limesurvey_sync_api_entityname($lss_entity_type, $ls_sid = FALSE) {

  // If $ls_sid value is 'prefix : return the prefix entity name : as an example :
  // 'limesurvey_sync_answer_'.
  $with_suffix = array('token', 'answer');
  $without_suffix = array('survey', 'response');

  if (in_array($lss_entity_type, $with_suffix) && (empty($ls_sid) || (!is_numeric($ls_sid) && $ls_sid != 'prefix'))) {
    return FALSE;
  }
  if ($ls_sid == 'prefix') {
    $ls_sid = '';
  }
  $sid_suffix = (in_array($lss_entity_type, $with_suffix)) ? "_{$ls_sid}" : '';
  if (in_array($lss_entity_type, array_merge($with_suffix, $without_suffix))) {
    // Return 'limesurvey_sync_survey' or 'limesurvey_sync_answer_123456', or
    // or 'limesurvey_sync_token_123456'.
    return "limesurvey_sync_{$lss_entity_type}{$sid_suffix}";
  }
  return FALSE;
}

/**
 * Return database table name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'limesurvey_sync_answer_123456_revision' for 'answer'
 * and '123456' and TRUE.
 *
 * @param $lss_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is FALSE.
 * @param $revision
 *   A boolean, if TRUE, return the revision table name. Default is FALSE.
 *
 * @return
 *   A string, the database table name.
 */
function limesurvey_sync_api_tablename($lss_entity_type, $ls_sid = FALSE, $revision = FALSE) {
  $entity_name = limesurvey_sync_api_entityname($lss_entity_type, $ls_sid);
  $revision_suffix = ($revision && $entity_name) ? '_revision' : '';
  return $entity_name . $revision_suffix;
}

/**
 * Implements hook_customizable_entities_overview_page_headers().
 *
 * Add synchronized columns to the overview page.
 */
function limesurvey_sync_customizable_entities_overview_page_headers($entity_type) {
  $headers = array();
  $info = entity_get_info($entity_type);
  $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : NULL;
  $ls_entitytypes = array(
    limesurvey_sync_api_entityname('survey'),
    limesurvey_sync_api_entityname('response'),
    limesurvey_sync_api_entityname('answer', $ls_sid),
    limesurvey_sync_api_entityname('token', $ls_sid),
  );
  if (in_array($entity_type, $ls_entitytypes)) {
    $lss_entity_type = $info['customizable entities']['lss_entity_type'];
    if ($lss_entity_type == 'token') {
      $headers['lss_uid'] = t('For user');
    }
    $headers['lss_sync'] = array(
      'data' => t('Sync status'),
      'field' => 'lss_sync',
      'options' => limesurvey_sync_api_lss_sync_status_list(),
    );
    $headers['lss_status'] = array(
      'data' => t('LS status'),
      'field' => 'lss_status',
    );

    $ls_type_funct = 'limesurvey_sync_api_' . $lss_entity_type . '_lss_status_list';
    if (function_exists($ls_type_funct)) {
      $ls_options = call_user_func($ls_type_funct);
      if (!empty($ls_options)) {
        $headers['lss_status']['options'] = $ls_options;
      }
    }
  }
  return $headers;
}

/**
 * Implements hook_customizable_entities_overview_page_values().
 *
 * Add formated synchronized cells to the overview page.
 */
function limesurvey_sync_customizable_entities_overview_page_values($entity_type, $entity) {
  $additional_cols = array();
  $info = entity_get_info($entity_type);
  $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : NULL;
  $ls_entitytypes = array(
    limesurvey_sync_api_entityname('survey'),
    limesurvey_sync_api_entityname('response'),
    limesurvey_sync_api_entityname('answer', $ls_sid),
    limesurvey_sync_api_entityname('token', $ls_sid),
  );
  if (in_array($entity_type, $ls_entitytypes)) {

    $lss_entity_type = $info['customizable entities']['lss_entity_type'];
    if ($lss_entity_type == 'token') {
      $additional_cols['lss_uid'] = theme('username', array('account' => user_load($entity->lss_uid)));
    }

    static $synchronize_set = array();
    if (empty($synchronize_set)) {
      // HERE : retrieve the synchronization set. Call it only one time.
      $all = 'all_skip_import';
      $synchronize_set = limesurvey_sync_synchronize($entity_type, $all);
    }
    $dp_key = $info['entity keys']['id'];
    $css = array(
      'css' => array(
        drupal_get_path('module', 'limesurvey_sync') . '/misc/limesurvey_sync.css',
      ),
    );

    $need_sync = FALSE;
    $need_sync_entity = (!empty($synchronize_set['need_sync']) && array_key_exists($entity->$dp_key, $synchronize_set['need_sync']));
    $need_sync_ref = (!empty($synchronize_set['need_sync_ref']) && array_key_exists($entity->$dp_key, $synchronize_set['need_sync_ref']));
    $lss_sync = $entity->lss_sync;
    $lss_status = $entity->lss_status;
    if ($need_sync_entity || $need_sync_ref) {
      $need_sync = TRUE;
      // Build the synchronized entity.
      if ($need_sync_entity) {
        //$sync_entity = $synchronize_set['need_sync'][$entity->$dp_key]['dp'];
        if (isset($synchronize_set['need_sync'][$entity->$dp_key]['diff']['lss_sync'])) {
          $lss_sync = $synchronize_set['need_sync'][$entity->$dp_key]['diff']['lss_sync'];
        }
        if (isset($synchronize_set['need_sync'][$entity->$dp_key]['diff']['lss_status'])) {
          $lss_status = $synchronize_set['need_sync'][$entity->$dp_key]['diff']['lss_status'];
        }
        //foreach ($synchronize_set['need_sync'][$entity->$dp_key]['diff'] as $key => $value) {
//  $sync_entity->$key = $value;
        //}
      }
      else {
        //$sync_entity = $synchronize_set['need_sync_ref'][$entity->$dp_key]['dp'];
      }
    }
    // lss_sync col. Display the sync status or 'To synchronize' if sync needed.
    $additional_cols['lss_sync'] = array(
      'data' => array(
        '#theme' => 'limesurvey_sync_field_label',
        '#field' => 'lss_sync',
        '#to_display' => $lss_sync,
        '#lss_entity_type' => $entity->lss_entity_type,
        '#ls_sid' => $entity->ls_sid,
        '#need_sync' => $need_sync,
        '#attached' => $css,
      ),
    );
    // lss_status_col.
    $additional_cols['lss_status'] = array(
      'data' => array(
        '#theme' => 'limesurvey_sync_field_label',
        '#field' => 'lss_status',
        '#to_display' => $lss_status,
        '#lss_entity_type' => $entity->lss_entity_type,
        '#ls_sid' => $entity->ls_sid,
        '#attached' => $css,
      ),
    );
  }
  return $additional_cols;
}

/**
 * Implements hook_customizable_entities_overview_page_links().
 *
 * Add synchronize link to the overview page.
 */
function limesurvey_sync_customizable_entities_overview_page_links($entity_type) {
  $links = array();
  $info = entity_get_info($entity_type);
  $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : NULL;
  $ls_entitytypes = array(
    // Do not synchronize surveys.
    //limesurvey_sync_api_entityname('survey'),
    limesurvey_sync_api_entityname('response'),
    limesurvey_sync_api_entityname('answer', $ls_sid),
    limesurvey_sync_api_entityname('token', $ls_sid),
  );
  if (in_array($entity_type, $ls_entitytypes)) {
    $links['synchronize'] = array(
      'title' => t('synchronize'),
      'suffix_path' => 'synchronize',
    );
  }
  return $links;
}

function limesurvey_sync_api_admin_block($form, $form_state, $entity, $entity_type, $langcode) {
  drupal_add_js('misc/form.js');
  drupal_add_js('misc/collapse.js');
  $info = entity_get_info($entity_type);
  $fieldsets = array();
  $fieldsets['admin'] = array(
    '#type' => 'fieldset',
    '#title' => t('Admin info'),
    '#access' => user_access(customizable_entities_permission_item('administer', $entity_type)),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
    '#description' => t('This block is visible only for administrator.'),
  );
  $fieldsets['admin']['sync'] = array(
    '#type' => 'fieldset',
    '#title' => t('Drupal info'),
    '#collapsed' => FALSE,
    '#collapsible' => TRUE,
  );
  $id_key = (!empty($info['entity keys']['name'])) ? $info['entity keys']['name'] : $info['entity keys']['id'];
  $view_sync = entity_view($entity_type, array($entity), 'admin_dp', $langcode);
  $fieldsets['admin']['sync']['datas'] = $view_sync[$entity_type][$entity->$id_key];
  // do not display "submited by X on date".
  if (isset($fieldsets['admin']['sync']['datas']['#theme']) && $fieldsets['admin']['sync']['datas']['#theme'] == 'customizable_entities') {
    $fieldsets['admin']['sync']['datas']['#theme'] = 'entity';
  }
  $fieldsets['admin']['ls'] = array(
    '#type' => 'fieldset',
    '#title' => t('LimeSurvey info'),
    '#collapsed' => FALSE,
    '#collapsible' => TRUE,
  );
  $view_ls = entity_view($entity_type, array($entity), 'admin_ls', $langcode);
  $fieldsets['admin']['ls']['datas'] = $view_ls[$entity_type][$entity->$id_key];
  if ($entity_type == 'limesurvey_sync_response') {
    // for response : no ls datas.
    unset($fieldsets['admin']['ls']);
  }
  // Do not display "submited by X on date".
  if (isset($fieldsets['admin']['ls']['datas']['#theme']) && $fieldsets['admin']['ls']['datas']['#theme'] == 'customizable_entities') {
    $fieldsets['admin']['ls']['datas']['#theme'] = 'entity';
  }
  return $fieldsets;
}

/**
 * Determines whether the given user has access to an entity.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 *
 * @see entity_type_supports()
 */
function limesurvey_sync_api_subentity_tab_access($op, $entity_type, $survey, $entity = NULL, $account = NULL ) {
  if (!is_object($survey)) {
    return FALSE;
  }
  // Check if the survey and the entity type match.
  $sub_entity_lss_entity_type = limesurvey_sync_api_retrieve_lss_entity_type_from_entityname($entity_type);
  $sub_entity_ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $sub_entity_lss_entity_type);
  if ($sub_entity_ls_sid != $survey->ls_sid) {
    // In the menu tree, make sure the answers and token entities are displayed
    // into the same menu tree that the corresponding survey.
    return FALSE;
  }
  else {
    return entity_access($op, $entity_type, $entity, $account);
  }
}

/**
 * Return the LS survey ID from the entity type name.
 *
 * As an example, for 'limsurvey_sync_answer_123456' and 'answer', returns
 * '123456'.
 *
 * @param $entity_type
 *   The entity type of the entity.
 * @param $lss_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 *
 * @return
 *   A numeric, the LS survey ID or FALSE if error or NULL if invalid
 *   parametters.
 */
function limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $lss_entity_type) {

  $with_suffix = array('token', 'answer');
  $without_suffix = array('survey', 'response');
  if (!empty($entity_type) && in_array($lss_entity_type, $with_suffix)) {
    $ls_sid = str_replace(limesurvey_sync_api_entityname($lss_entity_type, 'prefix'), '', $entity_type);
    if (empty($ls_sid) || !is_numeric($ls_sid)) {
      return FALSE;
    }
    return $ls_sid;
  }
  return NULL;
}

/*
 * Returns the LimeSurvey IDs that exists in the LimeSurvey site.
 *
 * @param $entity_type
 *   The LimeSurvey entity type. Correspond to a survey or an answer or a token
 *   with specific Limesurvey survey ID
 * @param $selection
 *   an array of LimeSurvey ID to returns or an array of Drupal LimeSurvey entity.
 *   Default is empty, returning all LimeSurvey ID.
 *
 * @return
 *   An array of LimeSurvey rows keyed by the LimeSurvey primary key or an empty
 *   array if no matching LimeSurvey entry. For each row, keys are the
 *   corresponding drupal key ('ls_token' instead of 'token') and values are the
 *   original ones (not converted) stored in LimeSurvey.
 */
function limesurvey_sync_api_existing_ls_ids($entity_type, $selection = array()) {
  $info = entity_get_info($entity_type);
  $lss_entity_type = (!empty($info['customizable entities']['lss_entity_type'])) ? $info['customizable entities']['lss_entity_type'] : NULL;
  $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : NULL;
  $ls_key = (!empty($info['entity keys']['ls key'])) ? $info['entity keys']['ls key'] : NULL;
  // Retrieve form LS.
  switch ($lss_entity_type) {
    case 'survey':
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
      $all_sids = limesurvey_sync_survey_properties('all');
      $ls_ids = ($all_sids) ? $all_sids : array();
      break;
    case 'answer':
    case 'token':
      // Adapt $selection
      if (!empty($selection) && is_array($selection)) {
        $selsected_ls_ids = array();
        foreach ($selection as $record) {
          if (is_object($record)) {
            // Selection is an entity.
            $selsected_ls_ids[] = $record->$ls_key;
          }
          else {
            $selsected_ls_ids[] = $record;
          }
        }
        // Overvrite selection.
        $selection = $selsected_ls_ids;
      }
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
      $ls_rows = limesurvey_sync_ls_api_retrieve_ls_row($lss_entity_type, $ls_sid, $selection);
      $ls_ids = ($ls_rows) ? $ls_rows : array();

  }
  return $ls_ids;
}

function limesurvey_sync_api_get_lss_survey($entity, array $options, $name, $entity_type) {
  $ls_sid = (!empty($entity->ls_sid)) ? $entity->ls_sid : NULL;
  if ($ls_sid) {
    $survey = limesurvey_sync_survey_get_survey_by_sid($ls_sid);
    return $survey->name;
  }
}

function limesurvey_sync_api_get_lss_page($entity, array $options, $name, $entity_type) {
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api');
  $info = entity_get_info($entity_type);
  $aid = $entity->{$info['entity keys']['ls key']};
  $page = FALSE;
  switch ($entity->lss_entity_type) {
    case 'survey':
      $aid = '';
      $page = 'admin';
      break;
    case 'token':
      $page = 'tokenview';
      break;
    case 'answer':
      $page = 'responseview';
      break;
  }
  if ($page) {
    return limesurvey_sync_ls_api_admin_url($entity->ls_sid, $page, '', TRUE, $aid);
  }
  return '';
}

function limesurvey_sync_api_get_lss_dp_filling_page($entity, array $options, $name, $entity_type) {
  $ls_sid = $entity->ls_sid;
  $survey = limesurvey_sync_survey_get_survey_by_sid($ls_sid);
  $path = 'lsform/' . str_replace('_', '-', $survey->name);
  $info = entity_get_info($entity_type);
  $lss_entity_type = (!empty($info['customizable entities']['lss_entity_type'])) ? $info['customizable entities']['lss_entity_type'] : '';
  if ($lss_entity_type && $lss_entity_type == 'token') {
    $dp_id = $entity->{$info['entity keys']['id']};
    if ($dp_id) {
      $path .= '/' . $dp_id;
    }
  }
  return $path;
}

function limesurvey_sync_api_get_lss_ls_filling_page($entity, array $options, $name, $entity_type) {
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
  // Absolute url.
  return limesurvey_sync_ls_api_answer_url($entity);
}

function limesurvey_sync_api_entity_metadata_lss_survey_properties_access($op, $property, $entity = NULL, $account = NULL, $entity_type) {
  return entity_access($op, $entity_type, $entity, $account);
}

function limesurvey_sync_api_permission_item($entity_type, $action) {
  switch ($action) {
    case 'dp2ls':
      return "ce synchronize dp2ls {$entity_type}";
      break;
    case 'launch':
      // $entity_type must be an anser or a token.
      $info = entity_get_info($entity_type);
      $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : NULL;
      $answer_entityname = limesurvey_sync_api_entityname('answer', $ls_sid);
      if ($answer_entityname) {
        return "ce launch {$answer_entityname}";
      }
      break;
  }
  return FALSE;
}

function limesurvey_sync_api_convert_object_to_array($object) {
  if (empty($object)) {
    return array();
  }
  elseif (is_array($object)) {
    return $object;
  }
  elseif (is_object($object)) {
    // Break the object association. Works on array.
    $array = array();
    foreach ($object as $key => $value) {
      $array[$key] = $value;
    }
    return $array;
  }
}

function limesurvey_sync_api_apply_changes(&$variable, $changes = array()) {
  if (!empty($changes) && is_array($changes)) {
    if (is_array($variable)) {
      foreach ($changes as $key => $value) {
        $variable[$key] = $value;
      }
    }
    elseif (is_object($variable)) {
      foreach ($changes as $key => $value) {
        $variable->$key = $value;
      }
    }
  }
}

function limesurvey_sync_api_rules_names($entity_type, $action, $way) {
  if ($action == 'sync') {
    return "rules_{$entity_type}_sync_{$way}";
  }
  if ($action == 'recent_token') {
    return "rules_{$entity_type}_recent_token";
  }
  if (in_array($action, array('import_user_datas', 'import_user_mail', 'import_user_name', 'create_user'))) {
    return "rules_{$entity_type}_{$action}_{$way}";
  }

  return FALSE;
}
