<?php

/**
 * @file
 * Functions and classes for the tokens entities.
 */

define('LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED', 1);
define('LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE', 2);
define('LIMESURVEY_SYNC_TOKEN_OPEN', 3);
define('LIMESURVEY_SYNC_TOKEN_SOON', 4);
define('LIMESURVEY_SYNC_TOKEN_EXPIRED', 5);
define('LIMESURVEY_SYNC_TOKEN_CLOSED', 6);
define('LIMESURVEY_SYNC_TOKEN_UNKNOWN', 7);

class LimesurveySyncSubentitiesToken extends LimesurveySyncSubentities {
  protected $info_class = 'LimesurveySyncEntityInfoToken';
}
/**
 * The entity info class for token entities.
 */
class LimesurveySyncEntityInfoToken extends LimesurveySyncSubentitiesEntityInfo {
  public $lss_entity_type = 'token';
  //public $crud_path = 'limesurvey/response';
  protected $primary_key = 'dp_tid';
  protected $date_col = TRUE;
  protected $ls_key = 'ls_tid';
  protected $ls_date_key = 'ls_completed';
  protected $label = 'token';
  protected $plural_label = 'tokens';
  // The token table stores the user id.
  // Alter the token access for 'own' to target lss_uid instead of uid.
  protected $entity_access_callback = 'limesurvey_sync_api_token_entity_access';

}

class LimesurveySyncInstallToken extends LimesurveySyncInstall {

  protected function schema_custom_fields($revision = FALSE) {

    $schema = array();
    $schema['lss_status'] = array(
      'type' => 'int',
      'unsigned' => TRUE,
      'not NULL' => TRUE,
      'default' => 1,
      'size' => 'tiny',
      'description' => "Status ID (1 and 2 for completed, 3 for open, and 4 to 7 for closed).",
    );
    $schema['lss_uid'] = array(
      'description' => "The {users}.uid this invitation is for.",
      'type' => 'int',
      'not null' => TRUE,
      'default' => 0,
    );
    $schema['lss_last_start'] = array(
      'type' => 'int',
      'not null' => TRUE,
      'default' => 0,
      'description' => 'The date when the answer has been started from the Drupal site.',
    );
    $schema = array_merge($schema, parent::schema_custom_fields($revision));

    return $schema;
  }

  protected function schema_ls_fields_known($field) {
    // Note : by default on drupal 'unsight' -> FALSE and 'not null' -> FALSE.
    $fields = array(
      'ls_tid' => array(
        'description' => 'Primary Key of the token table',
        'type' => 'int',
        'unsigned' => TRUE,
        'not null' => FALSE,
        'default' => NULL,
      ),
      // Defaults to NULL in order to avoid a brief period of potential
      // deadlocks on the index.
      'ls_participant_id' => array(
        'type' => 'varchar',
        'length' => 50,
        'default' => NULL,
        'description' => 'corresponding participant ID in the LimeSurvey participant panel.',
      ),
      'ls_firstname' => array(
        'type' => 'varchar',
        'length' => 40,
        'default' => NULL,
        'description' => 'First name.',
      ),
      'ls_lastname' => array(
        'type' => 'varchar',
        'length' => 40,
        'default' => NULL,
        'description' => 'Last name.',
      ),
      'ls_email' => array(
        'type' => 'text',
        'default' => NULL,
        'description' => "Email, 'OK' for a valid email.",
      ),
      'ls_emailstatus' => array(
        'type' => 'text',
        'default' => NULL,
        'description' => 'Email status.',
      ),
      'ls_token' => array(
        'type' => 'varchar',
        'length' => 35,
        'default' => NULL,
        'description' => 'Invitation token.',
      ),
      'ls_language' => array(
        'type' => 'varchar',
        'length' => 25,
        'default' => NULL,
        'description' => 'The LimeSurvey language.',
      ),
      'ls_blacklisted' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => NULL,
        'description' => 'Blacklisted entry.',
      ),
      'ls_sent' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => 'N',
        'description' => 'email sent.',
      ),
      'ls_remindersent' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => 'N',
        'description' => 'Reminder sent.',
      ),
      'ls_remindercount' => array(
        'type' => 'int',
        'unsigned' => TRUE,
        'default' => 0,
        'description' => 'Reminder count.',
      ),
      'ls_completed' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => 'N',
        'description' => "Completed status, 'N' for uncompleted.",
      ),
      'ls_usesleft' => array(
        'type' => 'int',
        'default' => 1,
        'description' => 'number of times a token can be used, default to one.',
      ),
      // Need to convert those fields values from datetime to timestamp.
      'ls_validfrom' => array(
        'type' => 'int',
        'default' => NULL,
        'description' => 'Token invitation valid from this date, unlimited if NULL.',
      ),
      'ls_validuntil' => array(
        'type' => 'int',
        'default' => NULL,
        'description' => 'Token invitation valid until this date, unlimited if NULL..',
      ),
      'ls_mpid' => array(
        'type' => 'int',
        'default' => NULL,
        'description' => 'mpid.',
      ),
    );

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal database schema of a LimeSurvey field.
   *
   * This fonction handles LimeSurvey token or answer fields that are specific to
   * a particular LimeSurvey survey. For the token table it corresponds to the
   * attribute fields, and for the answer table it corresponds to the questions
   * fields. The LimeSurvey datetime fields will be converted to timestamp on
   * Drupal.
   *
   * @param $type
   *   A string, 'token' or 'answer'.
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal schema corresponding to the provided LimeSurvey field or
   *   FALSE if the provided field is not a variable LimeSurvey field.
   */
  protected function schema_ls_fields_variables($field, $sid) {

    $attribute = array(
      'type' => 'varchar',
      'length' => 255,
      'not null' => FALSE,
      'default' => NULL,
    );
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    $fields = array();
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $fields[$attribute_field] = $attribute;
      $fields[$attribute_field]['description'] = $datas['description'];
    }

    return (isset($fields[$field])) ? $fields[$field] : FALSE;

  }

  protected function bundle_settings_default_order($extra, $view_mode = 'default') {
    $order = parent::bundle_settings_default_order($extra, $view_mode);
    $admin_ls = array('ls_tid', 'ls_token', 'ls_emailstatus', 'ls_participant_id', 'ls_blacklisted', 'ls_remindersent', 'ls_remindercount', 'ls_mpid');
    $admin_dp = array('lss_last_start');
    // Set all extra fileds to hidden for my custom view_modes.
    if (isset($extra[$this->entity_type][$this->entity_type]['display'])) {
      foreach ($extra[$this->entity_type][$this->entity_type]['display'] as $field => $extra_field_info) {
        // set admin_dp and admin_ls view_modes.
        //$type = (drupal_substr($field, 0, (drupal_strlen($this->ls_sid) + 3)) == 'ls_' . $this->ls_sid) ? 'answer_data' : ((drupal_substr($field, 0, 3) == 'ls_') ? 'ls_data' : 'entity_data');
        foreach (array('admin_dp', 'admin_ls') as $custom_view_mode) {
          // Do not overvrite settings from parent for dp_admin.
          $visibility = ($custom_view_mode == 'admin_ls' && ((isset($order[$field][$custom_view_mode]) && $order[$field][$custom_view_mode]) || in_array($field, $admin_ls))) || ($custom_view_mode == 'admin_dp' && ((isset($order[$field][$custom_view_mode]) && $order[$field][$custom_view_mode]) || in_array($field, $admin_dp)));
          $order[$field][$custom_view_mode] = $visibility;
          if ($visibility) {
            // If visible on admin_ls view_mode, hide it on default view mode.
            $order[$field]['default'] = FALSE;
          }
        }
      }
    }

    return $order;
  }

  protected function bundle_settings_default_order_ls_fields_known($view_mode = 'default') {
    $order = array();
    $order['ls_tid'] = array($view_mode => TRUE);
    $order['lss_uid'] = array($view_mode => TRUE);
    $order['ls_firstname'] = array($view_mode => TRUE);
    $order['ls_lastname'] = array($view_mode => TRUE);
    $order['ls_email'] = array($view_mode => TRUE);
    $order['ls_emailstatus'] = array($view_mode => FALSE);
    $order['ls_participant_id'] = array($view_mode => FALSE);
    $order['ls_token'] = array($view_mode => TRUE);
    $order['ls_language'] = array($view_mode => TRUE);
    $order['lss_last_start'] = array($view_mode => TRUE);
    $order['ls_usesleft'] = array($view_mode => TRUE);
    $order['ls_validfrom'] = array($view_mode => TRUE);
    $order['ls_validuntil'] = array($view_mode => TRUE);
    $order['ls_blacklisted'] = array($view_mode => TRUE);
    $order['ls_remindersent'] = array($view_mode => TRUE);
    $order['ls_remindercount'] = array($view_mode => TRUE);
    $order['ls_mpid'] = array($view_mode => FALSE);
    return $order;
  }

  protected function bundle_settings_default_order_ls_fields_variables($view_mode = 'default') {
    $order = array();
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($this->ls_sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $order[$attribute_field] = array($view_mode => TRUE);
    }
    return $order;
  }

}

class LimesurveySyncMetadataControllerToken extends LimesurveySyncMetadataController {

  protected function entityPropertyInfo_custom_fields($fields) {

    $fields = parent::entityPropertyInfo_custom_fields($fields);
    $fields['lss_status'] = array(
      'label' => t('DP Token status'),
      'description' => t('statut ID for the invitation (1 and 2 for completed, 3 for open, and 4 to 7 for closed).'),
      'type' => 'integer',
      //'validation callback' => 'entity_metadata_validate_integer_positive',
      'schema field' => 'lss_status',
      'options list' => 'limesurvey_sync_api_token_lss_status_list',
    );
    $fields['lss_uid'] = array(
      'label' => t('User'),
      'description' => t('The user this invitation is for.'),
      'type' => 'user',
      'getter callback' => 'entity_property_verbatim_get',
      //'getter callback' => 'entity_metadata_customizable_entities_get_properties',
      'setter callback' => 'entity_property_verbatim_set',
      'schema field' => 'lss_uid',
      //'setter_permission' => // Something specific ?
    );

    $fields['lss_last_start'] = array(
      'label' => t('Start date editing LS from DP'),
      'description' => t('The date when the LimeSurvey answer has been edited from the Drupal site.'),
      'type' => 'date',
      'getter callback' => 'entity_metadata_customizable_entities_get_date_property',
      'schema field' => 'lss_last_start',
    );
    return $fields;
  }

  /**
   * Return the corresponding Drupal entity_property_info of a LimeSurvey field.
   *
   * This fonction is used for generating the drupal hook_entity_property_info().
   * The main benefit of this fonction is to display the questions values instead
   * of the fieldname into Drupal Views. Handle common LimeSurvey fields of the
   * token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return An array of Drupal entity_property_info corresponding to the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic LimeSurvey
   *   field.
   */
  protected function entityPropertyInfo_ls_fields_known($field, $sid = NULL) {
    // Note : by default on drupal 'unsight' -> FALSE and 'not null' -> FALSE.
    $arg = array('%sid' => $sid);
    $fields = array(
      'ls_tid' => array(
        'label' => t('_ID, LimeSurvey token ID'),
        'description' => t('LimeSurvey token ID into the LimeSurvey database.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_tid',
        //'required' => TRUE,
      ),
      'ls_participant_id' => array(
        'label' => t('LS admin : Participant LimeSurvey ID'),
        'description' => t('Participant LimeSurvey ID into the LimeSurvey Software.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_participant_id',
      ),
      'ls_firstname' => array(
        'label' => t('LS admin : First name'),
        'description' => t('Token first name (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_firstname',
      ),
      'ls_lastname' => array(
        'label' => t('LS admin : Last name'),
        'description' => t('Token last name (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_lastname',
      ),
      'ls_email' => array(
        'label' => t('LS admin : Email'),
        'description' => t('Token email (survey#%sid)', $arg),
        'type' => 'text',
        'validation callback' => 'valid_email_address',
        'schema field' => 'ls_email',
      ),
      'ls_emailstatus' => array(
        'label' => t('LS admin : Email status'),
        'description' => t('Token email status (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_emailstatus',
      ),
      'ls_token' => array(
        'label' => t('LS admin : Token invitation'),
        'description' => t('Token invitation value (survey#%sid)', $arg),
        'type' => 'token',
        'schema field' => 'ls_token',
      ),
      'ls_language' => array(
        'label' => t('LS admin : LimeSurvey language'),
        'description' => t('LimeSurvey language (survey#%sid)', $arg),
        'type' => 'token',
        'options list' => 'limesurvey_sync_survey_ls_language_list',
        'schema field' => 'ls_language',
      ),
      'ls_blacklisted' => array(
        'label' => t('LS admin : Blacklisted entry'),
        'description' => t('Token blacklisted (survey#%sid)', $arg),
        'type' => 'token',
        'schema field' => 'ls_blacklisted',
      ),
      'ls_sent' => array(
        'label' => t('LS admin : Email sent'),
        'description' => t('Token email sent (survey#%sid)', $arg),
        'type' => 'date',
        'getter callback' => 'entity_metadata_customizable_entities_get_date_property',
        'schema field' => 'ls_sent',
      ),
      'ls_remindersent' => array(
        'label' => t('LS admin : Reminder sent'),
        'description' => t('Token reminder sent (survey#%sid)', $arg),
        'type' => 'date',
        'getter callback' => 'entity_metadata_customizable_entities_get_date_property',
        'schema field' => 'ls_remindersent',
      ),
      'ls_remindercount' => array(
        'label' => t('LS admin : Reminder count'),
        'description' => t('Token reminder count (survey#%sid)', $arg),
        'type' => 'int',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_remindercount',
      ),
      'ls_completed' => array(
        'label' => t('LS admin : Completed'),
        'description' => t('Token completed (survey#%sid)', $arg),
        'type' => 'date',
        'getter callback' => 'entity_metadata_customizable_entities_get_date_property',
        'schema field' => 'ls_completed',
      ),
      'ls_usesleft' => array(
        'label' => t('LS admin : Uses Left'),
        'description' => t('Token uses left (survey#%sid)', $arg),
        'type' => 'int',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_usesleft',
      ),
      // Need to convert those fields values from datetime to timestamp.
      'ls_validfrom' => array(
        'label' => t('LS admin : Valid from'),
        'description' => t('Token valid from (survey#%sid)', $arg),
        'type' => 'date',
        'schema field' => 'ls_validfrom',
      ),
      'ls_validuntil' => array(
        'label' => t('LS admin : Valid untill'),
        'description' => t('Token valid untill (survey#%sid)', $arg),
        'type' => 'date',
        'schema field' => 'ls_validuntil',
      ),
      'ls_mpid' => array(
        'label' => t('LS admin : Reminder count'),
        'description' => t('Token mpid (survey#%sid)', $arg),
        'type' => 'int',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_mpid',
      ),
    );

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal entity_property_info of a LimeSurvey field.
   *
   * This fonction is used for generating the drupal hook_entity_property_info().
   * The main benefit of this fonction is to display the questions values instead
   * of the fieldname into Drupal Views. Handle variable LimeSurvey fields of the
   * token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal entity_property_info corresponding to the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   */
  protected function entityPropertyInfo_ls_fields_variables($field, $sid) {
    $attribute = array(
      'type' => 'text',
    );

    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    $fields = array();
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $fields[$attribute_field] = $attribute;
      $fields[$attribute_field]['label'] = t('LS data !field : !attribute', array('!attribute' => $datas['description'], '!field' => $attribute_field));
      $fields[$attribute_field]['description'] = t('Token %attribute (survey #%sid)', array('%attribute' => $datas['description'], '%sid' => $sid));
      $fields[$attribute_field]['schema field'] = $attribute_field;
    }
    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }
}

class LimesurveySyncExtraFieldsControllerToken extends LimesurveySyncExtraFieldsController {
  public function __construct($type) {
    parent::__construct($type);
    // Disable some fields for edition
    $this->disable_extrafields[$this->entityType]['form'][] = 'lss_last_start';
  }
}

/**
 * Class managing building token entities.
 *
 * Building entity_info, entity_property_info, schema, entity default values and
 * updating entity table if needed. Take care of Drupal fields, LimeSurvey fixed
 * fields and Limesurver variables fields.
 */
class LimesurveySyncEntityToken extends LimesurveySyncEntity {
  public $ls_sid;
  public $lss_entity_type = 'token';
}

/**
 * The Controller for token entities
 */
class LimesurveySyncControllerToken extends LimesurveySyncController {
  public $lss_entity_type = 'token';

  protected function create_dp_fields_default_values($ls_sid = NULL) {
    $default_values = parent::create_dp_fields_default_values();
    $default_values['lss_status'] = LIMESURVEY_SYNC_TOKEN_UNKNOWN;
    $default_values['lss_uid'] = 0;
    $default_values['lss_last_start'] = 0;
    return $default_values;
  }

  /**
   * Return the corresponding Drupal default value of a LS known field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle common LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic (known)
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_known($field, $sid = NULL) {

    switch ($field) {
      case 'ls_tid':
      case 'ls_blacklisted':
      case 'ls_mpid':
      case 'ls_token':
      case 'ls_participant_id':
      case 'ls_validfrom':
      case 'ls_validuntil':
        return NULL;
        break;
      case 'ls_emailstatus':
        // @TODO: may need to edit this depending of the email value.
        return 'OK';
        break;
      case 'ls_language':
        // @TODO: may need to edit this depending of the drupal user language.
        module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
        $this_survey = limesurvey_sync_survey_properties($sid);
        return ($this_survey['default_language']) ? $this_survey['default_language'] : NULL;
        break;
      case 'ls_sent':
      case 'ls_remindersent':
      case 'ls_completed':
        return NULL;
        break;
      case 'ls_remindercount':
        return 0;
        break;
      case 'ls_usesleft':
        return 1;
        break;
      case 'ls_firstname':
      case 'ls_lastname':
        return NULL;
        break;
      case 'ls_email':
        return NULL;
        break;
    }


    return FALSE;
  }

  /**
   * Return the corresponding Drupal default value of a LS variable field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle variable LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_variables($field, $sid = NULL) {
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    $fields = array();
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $fields[$attribute_field] = NULL;
    }
    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Overridden.
   *
   * Add username token entities.
   */
  public function load($ids = array(), $conditions = array()) {

    $entities = parent::load($ids, $conditions);
    $uids = array();
    foreach ($entities as $id => $entity) {
      if (in_array('lss_uid', $this->entityInfo['schema_fields_sql']['base table'])) {
        $uids[$id] = $entity->lss_uid;
      }
    }
    if (in_array('lss_uid', $this->entityInfo['schema_fields_sql']['base table']) && !empty($uids)) {
      // Fetch name, picture, and data for these users.
      $user_fields = db_query("SELECT uid, name, picture, data FROM {users} WHERE uid IN (:uids)", array(':uids' => $uids))->fetchAllAssoc('uid');

      // Add these values back into the node objects.
      foreach ($uids as $id => $uid) {
        // set it as USERname instaed of name in order to avoid name conflict
        $entities[$id]->lss_username = $user_fields[$uid]->name;
        $entities[$id]->lss_picture = $user_fields[$uid]->picture;
        // set it as dataS instead of data in order to avoid name conflict
        $entities[$id]->lss_datas = $user_fields[$uid]->data;
      }
    }
    return $entities;
  }


  /**
   * Builds a structured array representing the entity's content.
   *
   * The content built for the entity will vary depending on the $view_mode
   * parameter.
   *
   * @param $entity
   *   An entity object.
   * @param $view_mode
   *   View mode, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   *
   * @return
   *   The renderable array.
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {

    //module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
    //$render_admin = limesurvey_sync_api_fields_render($this->lss_entity_type, $entity, $this->ls_sid);
    //$content['token_admin_datas'] = $render_admin + array(
//  '#theme' => 'limesurvey_sync_admin_data_inline',
    //);
    $build = parent::buildContent($entity, $view_mode, $langcode, $content);
    return $build;
  }
}

/**
 * The UI Controller for token entities
 */
class LimesurveySyncUIControllerToken extends LimesurveySyncUIController {
  public $lss_entity_type = 'token';
  protected $menu_weight = 1;
  protected $ls_sub_entity = TRUE;

  public function hook_menu() {
    // use the basc system
    // $items = EntityDefaultUIController::hook_menu();
    $items = parent::hook_menu();

    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%entity_object';
    $path_view = "{$this->entityInfo['customizable entities']['crud path']}/{$wildcard}";
    $id_count = count(explode('/', $path_view));
    $placeholder_view = $id_count - 1;

    $items["{$path_view}/launch"] = array(
      'title' => 'Launch',
      'page callback' => 'limesurvey_sync_api_token_launch',
      'page arguments' => array($this->entityType, $placeholder_view),
      'access callback' => 'limesurvey_sync_api_token_launch_access',
      'access arguments' => array($this->entityType, $placeholder_view),
      'type' => MENU_LOCAL_TASK,
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
      'file' => 'limesurvey_sync_token.class.inc',
      'weight' => 3,
    );

    // Page for creating token for drupal users.
    $overview_view = $this->entityInfo['customizable entities']['overview path'];
    $items["{$overview_view}/users"] = array(
      'title' => 'tokens for users',
      'page callback' => 'drupal_get_form',
      'page arguments' => array('limesurvey_sync_api_token_users_form', $this->entityType),
      'access arguments' => array(customizable_entities_permission_item('administer', $this->entityType)),
      'type' => MENU_LOCAL_TASK,
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
      'file' => 'limesurvey_sync_token.users.inc',
      'weight' => 3,
    );

    return $items;
  }
}

/**
 * Use lss_uid value for 'own' permission.
 *
 * Copy of the customizable_entities_entity_entity_access() function.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 *
 * @see entity_type_supports()
 */
function limesurvey_sync_api_token_entity_access($op, $entity = NULL, $account = NULL, $entity_type = NULL) {
  $own_field = 'lss_uid';
  return customizable_entities_entity_entity_access($op, $entity, $account, $entity_type, $own_field);
}

/**
 * Helper for the limesurvey_sync_ls_api_fields_convert_values() function.
 *
 * Manage converting datas from LimeSurvey to Drupal for common LimeSurvey fields.
 *
 * @param $field
 *   A string. The LimeSurvey field name.
 * @param $value
 *   A string or numeric or decimal or NULL. The LimeSurvey value to convert.
 * @param $way
 *   A string, 'ls2dp' or 'dp2ls' for synchronization way. Default is 'ls2pd'.
 *
 * @return
 *   A string or numeric or decimal or NULL : the Drupal converted value.
 */
function limesurvey_sync_token_fields_convert_values_known($field, $value, $way = 'ls2dp') {
  switch ($field) {
    case 'ls_tid':
    case 'ls_participant_id':
    case 'ls_firstname':
    case 'ls_lastname':
    case 'ls_email':
    case 'ls_emailstatus':
    case 'ls_token':
    case 'ls_language':
    case 'ls_remindercount':
    case 'ls_usesleft':
    case 'ls_mpid':
      // Sanitize.
      if ($way == 'ls2dp') {
        return (($value === NULL) ? NULL : check_plain($value));
      }
      else {
        // The entity_load function return null values as empty strings.
        return (($value === NULL || $value === '') ? NULL : check_plain($value));
      }
      break;
    case 'ls_blacklisted':
      if ($way == 'ls2dp') {
        return ($value == 'N') ? NULL : check_plain($value);
      }
      else {
        return (empty($value)) ? NULL : 'N';
      }
      break;
    case 'ls_sent':
    case 'ls_completed':
    case 'ls_remindersent':
      // Date or string
      $empty = (empty($value) || $value == 'N');
      if ($way == 'ls2dp') {
        if ($empty) {
          return NULL;
        }
        else {
          module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
          return limesurvey_sync_api_fixdate_ls2dp(check_plain($value));
        }
      }
      else {
        if ($empty) {
          return 'N';
        }
        else {
          // The completed date does not save seconds.
          return limesurvey_sync_api_fixdate_dp2ls(check_plain($value), 'Y-m-d H:i');
        }
      }
      break;
    case 'ls_validfrom':
    case 'ls_validuntil':
      if ($way == 'ls2dp') {
        // NULL or date.
        if (empty($value)) {
          return NULL;
        }
        else {
          module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
          return limesurvey_sync_api_fixdate_ls2dp(check_plain($value));
        }
        break;
      }
      else {
        // Need to convert those values from timestamp to datetime.
        if (empty($value)) {
          return NULL;
        }
        else {
          module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
          return limesurvey_sync_api_fixdate_dp2ls(check_plain($value));
        }
        break;
      }

    default:
      return FALSE;
  }

}

/**
 * Helper for the limesurvey_sync_ls_api_fields_convert_values() function.
 *
 * Manage converting datas from LimeSurvey to Drupal for variable LimeSurvey
 * fields.
 *
 * @param $sid
 *   A numeric. The LimeSurvey survey ID.
 * @param $field
 *   A string. The LimeSurvey field name.
 * @param $value
 *   A string or numeric or decimal or NULL. The LimeSurvey value to convert.
 *
 * @return
 *   A string or numeric or decimal or NULL : the Drupal converted value.
 */
function limesurvey_sync_token_fields_convert_values_variables($sid, $field, $value, $way) {
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $this_survey = limesurvey_sync_survey_properties($sid);
  $attributedescriptions = $this_survey['attributedescriptions'];
  foreach ($attributedescriptions as $attribute_field => $datas) {
    if ($attribute_field == $field) {
      return (($value === NULL) ? NULL : check_plain($value));
    }
  }

  return FALSE;
}

/**
 * Builder for the token form page.
 *
 * Not available yet. Will be usefull for 'dp2ls' synchronizations.
 *
 * @return
 *   An array, the form elements.
 */
function limesurvey_sync_token_form($form, &$form_state, $entity, $op = 'edit', $entity_type) {

  // Set the synchronization : Drupal -> LS
  $form['need_save_ls2dp'] = array(
    '#type' => 'value',
    '#value' => FALSE,
  );
  $form['need_save_dp2ls'] = array(
    '#type' => 'checkbox',
    '#title' => '<strong>' . t('Save the @entity_type in the LimeSurvey database', array('@entity_type' => $entity->lss_entity_type)) . '</strong>',
    '#default_value' => FALSE,
    '#description' => t('Warning : if checked, changes will be applied in your LimeSurvey site.'),
    '#weight' => -70,
    '#access' => user_access(limesurvey_sync_api_permission_item($entity_type, 'dp2ls')),
  );

  $form['ls_sid'] = array(
    '#type' => 'hidden',
    '#value' => $entity->ls_sid,
  );
  $form['lss_uid'] = array(
    '#type' => 'hidden',
    '#value' => $entity->lss_uid,
  );

  $form['lss_username'] = array(
    '#type' => 'textfield',
    '#title' => t('This invitation is for'),
    '#maxlength' => 60,
    '#autocomplete_path' => 'user/autocomplete',
    '#default_value' => !empty($entity->lss_username) ? $entity->lss_username : '',
    '#weight' => -1,
    '#description' => t('Leave blank for %anonymous.', array('%anonymous' => variable_get('anonymous', t('Anonymous')))),
    '#access' => user_access('access user profiles'),
  );


  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  $render = limesurvey_sync_api_fields_render($entity->lss_entity_type, $entity, $entity->ls_sid);
  $render_array = $render['for theme'];

  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.theme');

  $form['ls_fields'] = array(
    '#type' => 'fieldset',
    '#title' => t('Token datas'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#access' => user_access(limesurvey_sync_api_permission_item($entity_type, 'dp2ls')),
    //'#tree' => TRUE,
    //'#element_validate' => array('limesurvey_sync_answer_form_validate'),
  );

  foreach ($render_array as $delta => $ls_field) {
    $element = array();
    $ls_mandatory = $ls_help = $ls_weight = array();
    if (isset($ls_field['#ls_mandatory'])) {
      $ls_mandatory = array('#required' => ($ls_field['#ls_mandatory'] == 'Y'));
    }
    if (isset($ls_field['#ls_description'])) {
      $ls_help = array('#description' => $ls_field['#ls_description']);
    }
    if (isset($ls_field['#weight'])) {
      $ls_weight = array('#weight' => $ls_field['#weight']);
    }
    $fieldname = $ls_field['#ls_fieldname'];

    $value = $entity->$fieldname;
    $input_name = FALSE;
    switch ($fieldname) {
      case 'ls_tid':
        $element = array(
          '#type' => 'textfield',
          '#title' => $ls_field['#ls_label'],
          '#default_value' => (string) $value,
          '#disabled' => TRUE,
        );
        break;
      case 'ls_participant_id':
      case 'ls_firstname':
      case 'ls_lastname':
      case 'ls_email':
      case 'ls_token':
      case 'ls_remindercount':
      case 'ls_usesleft':
        $element = array(
          '#type' => 'textfield',
          '#title' => $ls_field['#ls_label'],
          '#default_value' => $value,
          //'#size' => 50,
        );
        break;

        // Dates.
      case 'ls_completed':
      case 'ls_sent':
      case 'ls_remindersent':

        // Bellow :#states don't work.
        //$fieldname_checkbox = $ls_field['#ls_fieldname'] . '_checkbox';
        //$input_name = $fieldname_checkbox;
        //$form['ls_fields'][$fieldname_checkbox] = array(
        //'#type' => 'checkbox',
        //'#title' => $ls_field['#ls_label'],
        //'#default_value' => (!empty($value)),
        //'#weight' => (((int) $ls_field['#weight']) - 0.5),
        //);

      case 'ls_validfrom':
      case 'ls_validuntil':

        $element = array(
          '#type' => (module_exists('date_popup')) ? 'date_popup' : 'textfield',
          // Use 'timepicker' option provided by the Date Popup Timepicker module.
          // #timpicker propertry itself is provided by the Date Popup module.
          //'#timepicker' => 'timepicker',
          // New #timepicker_options property handled by the Date Popup Timepicker
          // module and lets to customize timepicker options.
          //'#timepicker_options' => array(
          //  'rows' => 6,
          //  'minutes' => array(
          //    'starts' => 0,
          //    'ends' => 56,
          //    'interval' => 4,
          //  ),
          //  'showCloseButton' => TRUE,
          //  'closeButtonText' => t('Close'),
          //),
          '#title' => $ls_field['#ls_label'],
          '#default_value' => (!empty($value)) ? format_date($value, 'custom', 'Y-m-d H:i:s') : NULL,
        );
        if ($input_name) {
          $element['#state'] = array('visible' => array(
              ':input[name="' . $input_name . '"]' => array('checked' => TRUE),
            ));
        }
        break;
      case 'ls_mpid':
      case 'ls_blacklisted': // Not sur how manage it. Arbitrary disabled from UI.
      case 'ls_emailstatus':
        $element = array(
          '#type' => 'value',
          '#value' => $value,
        );
        break;
      case 'ls_language':
        // Set the ls_language value.
        module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
        $options_ls_lang = limesurvey_sync_api_get_ls_langs($entity->ls_sid);
        $default_ls_lang = (!empty($entity->ls_language)) ? $entity->ls_language : limesurvey_sync_api_set_ls_lang_from_current_dp_lang($entity->ls_sid);
        if (count($options_ls_lang) <= 1) {
          // Only one language, don't display it.
          $element = array(
            '#type' => 'value',
            '#value' => $default_ls_lang,
          );
        }
        else {
          // Propose to choose.
          $element = array(
            '#type' => 'select',
            '#title' => t('Select a language for the form'),
            '#options' => $options_ls_lang,
            '#default_value' => $default_ls_lang,
            //'#access' => ((empty($options['ls_hide_lang'])) ? 1 : 0),
          );
        }
        break;
      default:
        // ls_attribute_1

        $element = array(
          '#type' => 'textfield',
          '#title' => $ls_field['#ls_label'],
          '#default_value' => (string) $value,
          //'#size' => 50,
        );
    }

    if (!empty($element)) {
      $element += $ls_mandatory;
      $element += $ls_help;
      $element += $ls_weight;
      $form['ls_fields'][$ls_field['#ls_fieldname']] = $element;
    }
  }

  return $form;
}

/**
 * Builder for the token form page validation.
 *
 * Not available yet. Will be usefull for 'dp2ls' synchronizations.
 *
 * @return
 *   An array, the form elements.
 */
function limesurvey_sync_token_form_validate($form, &$form_state) {
  // Debug : don't save.
  //form_set_error('', t('Token is not available for saving, coding in progress...'));
  // Check date
  $values = $form_state['values'];
  $date_fields = array('ls_completed', 'ls_validfrom', 'ls_validuntil');
  foreach ($date_fields as $date_field) {
    $empty = (empty($values[$date_field]) || ($date_field == 'ls_completed' && $values[$date_field] == 'N'));
    if (!$empty && strtotime($values[$date_field]) === FALSE) {
      form_set_error($date_field, t('You have to specify a valid date.'));
    }
  }
  if (!empty($values['ls_email'])) {
    if (!valid_email_address($values['ls_email'])) {
      form_set_error('ls_email', t('The email is not valid.'));
    }
    else {
      if ($account = user_load_by_mail($values['ls_email'])) {
        // A email is provided, check if it matchs the author.
        if (!empty($values['lss_username']) && $values['lss_username'] != $account->name) {
          form_set_error('ls_email', t('The user this invitation is for does not match the email address.'));
        }
      }
      else {
        // The token is for a user with a distinct email that the drupal one,
        // and this mail not exists in drupal.
        drupal_set_message('The email you defined does not match the user email.', 'warning');
      }
    }
  }
}

function limesurvey_sync_token_form_submit($form, &$form_state) {
  // Fix date
  $values = $form_state['values'];
  $date_fields = array('ls_completed', 'ls_validfrom', 'ls_validuntil');
  foreach ($date_fields as $date_field) {
    $form_state['values'][$date_field] = (!empty($values[$date_field])) ? strtotime($values[$date_field]) : NULL;
  }
  if (!empty($values['ls_email'])) {
    $form_state['values']['ls_emailstatus'] = 'OK';
  }

  // A user might assign the node author by entering a user name in the node
  // form, which we then need to translate to a user ID.
  if (isset($form_state['values']['lss_username'])) {
    if ($account = user_load_by_name($form_state['values']['lss_username'])) {
      $form_state['values']['lss_uid'] = $account->uid;
    }
    else {
      $form_state['values']['lss_uid'] = 0;
    }
  }

}


/**
 * List of the token lss_status possible values.
 *
 * Used for compatibility with Views and Rules modules, throught
 * hook_entity_property_info().
 *
 * @return
 *   An array, the keys are the lss_status values and values are the human
 *   corresponding display message.
 */
function limesurvey_sync_api_token_lss_status_list() {
  $status = array(
    LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED,
    LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE,
    LIMESURVEY_SYNC_TOKEN_OPEN,
    LIMESURVEY_SYNC_TOKEN_SOON,
    LIMESURVEY_SYNC_TOKEN_EXPIRED,
    LIMESURVEY_SYNC_TOKEN_CLOSED,
    LIMESURVEY_SYNC_TOKEN_UNKNOWN,
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($status as $value) {
    $list[$value] = limesurvey_sync_api_display_field('token', 'lss_status', $value);
  }
  return $list;
}

/**
 * Return a drupal LS field for display.
 *
 * As an example, return timestamp as formated date, LS language code as human
 * language, etc...
 *
 * @param $field
 *   A string. The Drupal ls_field to display (ex : 'startlanguage').
 * @param $value
 *   A string or any. corresponding Drupal LS field to display (ex:
 *   'de-informal').
 * @param $return
 *   A string, 'value' or 'help' or 'css'. Default to 'value'.
 * @param $ls_sid
 *   A numeric, the LS survey ID.
 *
 * @return
 *   A string. The value ready to be displayed or an help text or a css class.
 */
function limesurvey_sync_token_display_field($field, $value, $return = 'value', $ls_sid = FALSE) {

  if (in_array($field, array(
    'ls_completed',
    'ls_language',
    'ls_sent',
    'ls_remindersent',
    'ls_blacklisted',
    'ls_validfrom',
    'ls_validuntil',
  ))) {
    if (in_array($return, array(
      'help',
      'css',
    ))) {
      return '';
      // No need to explicit css or help.
    }
    else {
      switch ($field) {
        case 'ls_completed':
          if ($value == '') {
            return t('Not completed');
          }
          return format_date($value);
          break;
        case 'ls_language':
          return limesurvey_sync_api_display_field('token', 'ls_lang', $value, $return, $ls_sid);
          break;
        case 'ls_sent':
        case 'ls_remindersent':
          if ($value == '') {
            return t('No');
          }
          else {
            return (!is_numeric($value)) ? $value : format_date($value);
          }
          break;
        case 'ls_blacklisted':
          if ($value == '') {
            return t('Not blaklisted');
          }
          break;
        case 'ls_validfrom':
        case 'ls_validuntil':
          if ($value == '') {
            return t('any time');
          }
          return format_date($value);
          break;
      }
    }
  }

  switch ($field) {
    case 'lss_status':
      switch ($value) {
        case LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED:
          switch ($return) {
            case 'help':
              t('The token invitation is completed and closed.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-open';
            default:
              return t('Completed');
          }
          break;

        case LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE:
          switch ($return) {
            case 'help':
              return t('The token invitation is completed but reusable.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-completed-reusable';
            default:
              return t('Completed reusable');
          }
          break;

        case LIMESURVEY_SYNC_TOKEN_OPEN:
          switch ($return) {
            case 'help':
              return t("The token invitation is open.");
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-open';
            default:
              return t('Open');
          }
          break;

        case LIMESURVEY_SYNC_TOKEN_SOON:
          switch ($return) {
            case 'help':
              return t('The token invitation will be open in the future.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-soon';
            default:
              return t('Not open yet');
          }
          break;

        case LIMESURVEY_SYNC_TOKEN_EXPIRED:
          switch ($return) {
            case 'help':
              return t('The token invitation has expired.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-expired';
            default:
              return t('Expired');
          }

        case LIMESURVEY_SYNC_TOKEN_CLOSED:
          switch ($return) {
            case 'help':
              return t('The token invitation is uncompleted and closed.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-closed';
            default:
              return t('Closed');
          }
          break;

        default:
          switch ($return) {
            case 'help':
              return t('Unknown token status.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-unknown';
            default:
              return t('Unknown');
          }
      }
      break;

    case 'lss_last_start':
      if (empty($value)) {
        return '';
      }
      else {
        return format_date($value);
      }
      break;
  }

  return $value;
}

function limesurvey_sync_api_token_launch($entity_type, $token) {
  $survey_type_url = str_replace('_', '-', $token->lss_survey);
  $info = entity_get_info($entity_type);
  $uri = "lsform/{$survey_type_url}/{$token->{$info['entity keys']['id']}}";
  drupal_goto($uri);
}

/**
 * Provide the token entity status.
 *
 * Status are 'completed_closed', 'completed reusale', 'open', 'soon',
 * 'expired' and 'unknown'.
 *
 * @param $entity
 *   The entity object.
 *
 * @return
 *   A numeric defined as a constant corresponding to a defined LS stage.
 */
function limesurvey_sync_api_token_lss_status($entity_type, $entity) {

  if (is_array($entity)) {
    $entity = (object) $entity;
  }
  if (isset($entity->ls_completed) && $entity->ls_completed !== NULL) {
    return ($entity->ls_usesleft < 1) ? LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED : LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE;
  }
  elseif ($entity->ls_usesleft < 1) {
    return LIMESURVEY_SYNC_TOKEN_CLOSED;
  }
  elseif (($entity->ls_validfrom === NULL || $entity->ls_validfrom <= REQUEST_TIME) && ($entity->ls_validuntil === NULL || $entity->ls_validuntil > REQUEST_TIME)) {
    return LIMESURVEY_SYNC_TOKEN_OPEN;
  }
  elseif ($entity->ls_validfrom !== NULL && $entity->ls_validfrom > REQUEST_TIME) {
    return LIMESURVEY_SYNC_TOKEN_SOON;
  }
  elseif ($entity->ls_validuntil !== NULL && $entity->ls_validuntil < REQUEST_TIME) {
    return LIMESURVEY_SYNC_TOKEN_EXPIRED;
  }
  return LIMESURVEY_SYNC_TOKEN_UNKNOWN;
}

/**
 * Access for the token launch tab, that will redirect to the filling survey page..
 *
 * @param $token
 *   A token entity.
 * @param $entity_type
 *   The associated entity type.
 *
 * @return
 *   A boolean, TRUE if access granted.
 */
function limesurvey_sync_api_token_launch_access($entity_type, $token) {
  if (is_object($token)) {
    // Check permissions and if survey exists
    $ls_sid = $token->ls_sid;
    if (!empty($ls_sid) && $survey = entity_load_single('limesurvey_sync_survey', $token->lss_survey) && user_access(limesurvey_sync_api_permission_item($entity_type, 'launch'))) {
      // Chek if the token is open.
      $lss_status = limesurvey_sync_api_token_lss_status($entity_type, $token);
      if ($token->lss_sync == LIMESURVEY_SYNC_SYNCHRONIZED && in_array($lss_status, array(LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE, LIMESURVEY_SYNC_TOKEN_OPEN))) {
        // Token is open, check for the user
        // Allow hight level users (administer survey) to fill survey survey for
        // another user.
        if (user_access(customizable_entities_permission_item('administer', 'limesurvey_sync_survey'))) {
          return TRUE;
        }
        else {
          global $user;
          if ($user->uid && $user->uid == $token->lss_uid) {
            return TRUE;
          }
        }
      }
    }
  }
  return FALSE;
}


function limesurvey_sync_api_token_retreive_recent_open_token($token_entityname, $uid) {
  $token = FALSE;
  $token_info = entity_get_info($token_entityname);
  if ($token_entityname && $token_info) {
    $token_id_field = $token_info['entity keys']['id'];
    $result = db_select($token_info['base table'], 't')
      ->fields('t', array($token_id_field))
      ->condition('lss_uid', $uid)
      ->condition('status', 1)
      ->condition('lss_sync', LIMESURVEY_SYNC_SYNCHRONIZED)
      ->condition('ls_usesleft', 0, '>')
      ->condition(db_or()->isNull('ls_validfrom')->condition('ls_validfrom', REQUEST_TIME, '>'))
      ->condition(db_or()->isNull('ls_validuntil')->condition('ls_validuntil', REQUEST_TIME, '<'))
      ->orderBy('changed', 'DESC')
      ->range(0, 1)
      ->addMetaData('account', user_load(1)) // Run the query as user 1.
      ->execute();
    if (!empty($result)) {
      foreach ($result as $record) {
        $token_id = $record->$token_id_field;
        $token = entity_load_single($token_entityname, $token_id);
      }

    }
  }
  return $token;
}
